import click

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    DREMIO_DATABASE_TYPE,
    MYSQL_DB_TYPE,
    ORACLE_DB_TYPE,
    TERADATA_DB_TYPE,
    TRANSACTIONAL_CONNECTION_TYPE,
    TRANSACTIONAL_WAREHOUSE_TYPE,
)


class TransactionalOnboardingService(CollectorValidationService, BaseOnboardingService):
    def __init__(self, config: Config, command_name: str, **kwargs):
        super().__init__(config, command_name=command_name, **kwargs)

    @manage_errors
    def onboard_transactional_db(self, **kwargs) -> None:
        """
        Onboard a Transactional DB connection by validating and adding a connection.
        """
        ssl_options = self.load_ssl_options(options=kwargs)
        if ssl_options:
            kwargs.setdefault("connection_settings", {})["ssl_options"] = ssl_options

        kwargs["connection_type"] = TRANSACTIONAL_CONNECTION_TYPE
        kwargs["warehouse_type"] = TRANSACTIONAL_WAREHOUSE_TYPE

        # These dbTypes have been promoted to their own connection type,
        # instead of being a 'transactional-db' connection type
        promoted_subtypes = [TERADATA_DB_TYPE, DREMIO_DATABASE_TYPE, ORACLE_DB_TYPE, MYSQL_DB_TYPE]
        if kwargs.get("dbType") in promoted_subtypes:
            kwargs["connection_type"] = kwargs.get("dbType")
            kwargs["warehouse_type"] = kwargs.get("dbType")

        # Finds all v2 validations for this connection and runs each one.
        # If they all pass, return a temp credentials key
        # If the --skip-validations flag has been used, no validations are run and just a
        # temp key is returned
        key = self.test_new_credentials(**kwargs)

        # Add the connection
        if key:
            self.add_connection(key, **kwargs)
        else:
            click.Abort()



from typing import List, Optional

import click
import click_config_file

import montecarlodata.settings as settings
from montecarlodata.common import create_mc_client
from montecarlodata.common.commands import DISAMBIGUATE_DC_OPTIONS
from montecarlodata.common.resources import CloudResourceService
from montecarlodata.integrations.configure.bi.bi import BiService
from montecarlodata.integrations.info.status import OnboardingStatusService
from montecarlodata.integrations.keys import IntegrationKeyScope, IntegrationKeyService
from montecarlodata.integrations.onboarding.bi.reports import ReportsOnboardingService
from montecarlodata.integrations.onboarding.data_lake.databricks import (
    DEFAULT_SECRET_NAME,
    DEFAULT_SECRET_SCOPE,
    DatabricksOnboardingService,
)
from montecarlodata.integrations.onboarding.data_lake.events import (
    EventsOnboardingService,
)
from montecarlodata.integrations.onboarding.data_lake.glue_athena import (
    GlueAthenaOnboardingService,
)
from montecarlodata.integrations.onboarding.data_lake.hive import HiveOnboardingService
from montecarlodata.integrations.onboarding.data_lake.presto import (
    PrestoOnboardingService,
)
from montecarlodata.integrations.onboarding.data_lake.spark import (
    SPARK_BINARY_MODE_CONFIG_TYPE,
    SPARK_DATABRICKS_CONFIG_TYPE,
    SPARK_HTTP_MODE_CONFIG_TYPE,
    SparkOnboardingService,
)
from montecarlodata.integrations.onboarding.etl.adf import (
    AzureDataFactoryOnboardingService,
)
from montecarlodata.integrations.onboarding.etl.airflow import AirflowOnboardingService
from montecarlodata.integrations.onboarding.etl.dbt_cloud import (
    DbtCloudOnboardingService,
)
from montecarlodata.integrations.onboarding.etl.fivetran import (
    FivetranOnboardingService,
)
from montecarlodata.integrations.onboarding.etl.informatica import InformaticaOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    AZURE_DEDICATED_SQL_POOL_TYPE,
    AZURE_SQL_DATABASE_TYPE,
    CONNECTION_TO_WAREHOUSE_TYPE_MAP,
    DATABRICKS_DELTA_CONNECTION_TYPE,
    DATABRICKS_METASTORE_CONNECTION_TYPE,
    DATABRICKS_METASTORE_SQL_WAREHOUSE_CONNECTION_TYPE,
    DATABRICKS_SQL_WAREHOUSE_CONNECTION_TYPE,
    DREMIO_DATABASE_TYPE,
    GLUE_CONNECTION_TYPE,
    GQL_TO_FRIENDLY_CONNECTION_MAP,
    HIVE_MYSQL_CONNECTION_TYPE,
    HIVE_S3_CONNECTION_TYPE,
    MARIADB_DB_TYPE,
    MOTHERDUCK_DATABASE_TYPE,
    MYSQL_DB_TYPE,
    ORACLE_DB_TYPE,
    POSTGRES_DB_TYPE,
    PRESTO_S3_CONNECTION_TYPE,
    SAP_HANA_DATABASE_TYPE,
    SECRETS_MANAGER_CREDENTIAL_MECHANISM,
    SELF_HOSTING_MECHANISMS,
    SQL_SERVER_DB_TYPE,
    TERADATA_DB_TYPE,
)
from montecarlodata.integrations.onboarding.operations.connection_ops import (
    ConnectionOperationsService,
)
from montecarlodata.integrations.onboarding.self_hosted_credentials import (
    SelfHostedCredentialOnboardingService,
)
from montecarlodata.integrations.onboarding.streaming.streamings import (
    StreamingOnboardingService,
)
from montecarlodata.integrations.onboarding.transactional import (
    TransactionalOnboardingService,
)
from montecarlodata.integrations.onboarding.vector import (
    VectorDbOnboardingService,
)
from montecarlodata.integrations.onboarding.warehouse.warehouses import (
    WarehouseOnboardingService,
)
from montecarlodata.tools import (
    AdvancedOptions,
    add_common_options,
    convert_empty_str_callback,
    convert_uuid_callback,
    validate_json_callback,
)

# Shared command verbiage
METADATA_VERBIAGE = "For metadata"
QL_VERBIAGE = "For query logs"
SQL_VERBIAGE = "For health queries"
EVENT_VERBIAGE = (
    "For tracking data freshness and volume at scale. "
    "Requires s3 notifications to be configured first"
)
REGION_VERBIAGE = "If not specified the region the collector is deployed in is used"
WAREHOUSE_VERBIAGE = "For metadata, query logs and metrics"
LIGHTWEIGHT_VERBIAGE = "For metadata, and custom SQL monitors"
BI_VERBIAGE = "For reports"
PASSWORD_VERBIAGE = f"If you prefer a prompt (with hidden input) enter {settings.SHOW_PROMPT_VALUE}"
RESOURCE_VERBIAGE = "This can be helpful if the resources are in different accounts"
CONNECTION_ID_VERBIAGE = "ID for the connection."


# Options shared across commands
def role_options(required: bool = True) -> List:
    return [
        click.option(
            "--role",
            help="Assumable role ARN to use for accessing AWS resources.",
            required=required,
        ),
        click.option(
            "--external-id",
            help="An external id, per assumable role conditions.",
            required=False,
        ),
    ]


# Name is used for the creation of a warehouse that will contain the connection.
def warehouse_create_option(required: bool = False, default: Optional[str] = None) -> List:
    return _warehouse_option(
        help="Friendly name for the created integration (e.g. warehouse). Name must be unique.",
        required=required,
        default=default,
    )


# Name is used to identify an existing warehouse that will contain the connection.
def warehouse_select_option(required: bool = False, default: Optional[str] = None) -> List:
    return _warehouse_option(
        help="Friendly name of the warehouse which the connection will belong to.",
        required=required,
        default=default,
    )


def _warehouse_option(help: str, required: bool = False, default: Optional[str] = None) -> List:
    return [click.option("--name", help=help, required=required, default=default)]


# Name is used to give a distinctive name to the etl connection.
def etl_select_option(required: bool = False, default: Optional[str] = None) -> List:
    return _etl_option(
        help="Friendly name for the etl connection.",
        required=required,
        default=default,
    )


def _etl_option(help: str, required: bool = False, default: Optional[str] = None) -> List:
    return [click.option("--name", help=help, required=required, default=default)]


def port_create_option(default_port: int) -> List:
    return [
        click.option(
            "--port",
            help="HTTP port.",
            default=default_port,
            type=click.INT,
            show_default=True,
        )
    ]


WAREHOUSE_OPTIONAL_CREATE_OPTIONS = [
    click.option(
        "--create-warehouse",
        help="Create a new warehouse with this connection",
        type=click.BOOL,
        default=True,
        required=False,
    )
]
S3_OPTIONS = [
    click.option("--bucket", help="S3 Bucket where query logs are contained.", required=True),
    click.option("--prefix", help="Path to query logs.", required=True),
    *role_options(required=False),
]

BASIC_DATABASE_OPTIONS = [
    click.option("--host", help="Hostname.", required=True),
    click.option("--user", help="Username with access to the database.", required=True),
    click.option(
        "--password",
        help=f"User's password. {PASSWORD_VERBIAGE}.",
        required=True,
        cls=AdvancedOptions,
        prompt_if_requested=True,
    ),
]

DATABASE_OPTIONS = [
    *BASIC_DATABASE_OPTIONS,
    click.option("--database", help="Name of database/site.", required=True),
]

CONNECTION_OPTIONS = [
    click.option(
        "--connection-id",
        help=CONNECTION_ID_VERBIAGE,
        required=True,
        type=click.UUID,
        callback=convert_uuid_callback,
    )
]

SSL_OPTIONS = [
    click.option(
        "--ssl-ca",
        help="Path to the file that contains a PEM-formatted CA certificate.",
        required=False,
        type=click.Path(dir_okay=False, exists=True),
        cls=AdvancedOptions,
        mutually_exclusive_options=["ssl_disabled"],
    ),
    click.option(
        "--ssl-cert",
        help="Path to the file that contains a PEM-formatted client certificate.",
        required=False,
        type=click.Path(dir_okay=False, exists=True),
        cls=AdvancedOptions,
        mutually_exclusive_options=["ssl_disabled"],
    ),
    click.option(
        "--ssl-key",
        help="Path to the file that contains a PEM-formatted private key for the "
        "client certificate.",
        required=False,
        type=click.Path(dir_okay=False, exists=True),
        cls=AdvancedOptions,
        required_with_options=["ssl_cert"],
        mutually_exclusive_options=["ssl_disabled"],
    ),
    click.option(
        "--ssl-disabled",
        help="A boolean value that disables usage of TLS.",
        required=False,
        type=click.BOOL,
        cls=AdvancedOptions,
        mutually_exclusive_options=["ssl_ca", "ssl_cert", "ssl_key"],
    ),
    click.option(
        "--ssl-key-password",
        help=f"The password for the client certificate private key. {PASSWORD_VERBIAGE}.",
        required=False,
        cls=AdvancedOptions,
        prompt_if_requested=True,
        required_with_options=["ssl_cert"],
    ),
    click.option(
        "--ssl-verify-cert",
        help="Set to true to check the server certificate’s validity.",
        required=False,
        type=click.BOOL,
        cls=AdvancedOptions,
        required_with_options=["ssl_cert"],
    ),
    click.option(
        "--ssl-verify-identity",
        help="Set to true to check the server’s identity.",
        required=False,
        type=click.BOOL,
        cls=AdvancedOptions,
        required_with_options=["ssl_ca"],
    ),
    click.option(
        "--skip-cert-verification",
        help="Skip SSL certificate verification.",
        required=False,
        is_flag=True,
        cls=AdvancedOptions,
        mutually_exclusive_options=["ssl_verify_cert", "ssl_verify_identity"],
    ),
]

ONBOARDING_CONFIGURATION_OPTIONS = [
    *DISAMBIGUATE_DC_OPTIONS,
    click.option(
        "--skip-validation",
        is_flag=True,
        help="Skip all connection tests.",
        cls=AdvancedOptions,
        mutually_exclusive_options=["validate_only"],
    ),
    click.option(
        "--validate-only",
        is_flag=True,
        help="Run connection tests without adding.",
        cls=AdvancedOptions,
        mutually_exclusive_options=["skip_validation"],
    ),
    click.option(
        "--auto-yes",
        is_flag=True,
        help="Skip any interactive approval.",
        default=False,
        show_default=True,
    ),
]

NETWORK_OPTIONS = [
    click.option(
        "--test-network-only",
        "skip_permission_tests",
        is_flag=True,
        default=False,
        show_default=True,
        cls=AdvancedOptions,
        mutually_exclusive_options=["skip_validation"],
        help="Skip any permission tests. Only validates network connection between the "
        "collector and resource can be established.",
    )
]

BI_OPTIONS = [
    click.option(
        "--verify-ssl/--no-verify-ssl",
        "verify_ssl",
        required=False,
        default=True,
        show_default=True,
        help="Whether to verify the SSL connection (uncheck for self-signed certs).",
    )
]

BASE_DATABRICKS_OPTIONS = [
    click.option("--databricks-workspace-url", help="Databricks workspace URL.", required=True),
    click.option(
        "--databricks-token",
        help=f"Databricks access token. {PASSWORD_VERBIAGE}.",
        required=False,
        cls=AdvancedOptions,
        prompt_if_requested=True,
    ),
]

DATABRICKS_OPTIONS = [
    *BASE_DATABRICKS_OPTIONS,
    click.option("--databricks-workspace-id", help="Databricks workspace ID.", required=True),
    click.option("--databricks-cluster-id", help="Databricks cluster ID.", required=True),
]

DATABRICKS_SQL_WAREHOUSE_OPTIONS = [
    *BASE_DATABRICKS_OPTIONS,
    click.option("--databricks-warehouse-id", help="Databricks warehouse ID.", required=True),
    click.option(
        "--databricks-client-id",
        help="Databricks OAuth Client ID.",
        required=False,
        cls=AdvancedOptions,
        mutually_exclusive_options=["databricks_token"],
        required_with_options=["databricks_client_secret"],
    ),
    click.option(
        "--databricks-client-secret",
        help=f"Databricks OAuth Client Secret. {PASSWORD_VERBIAGE}.",
        required=False,
        cls=AdvancedOptions,
        prompt_if_requested=True,
        mutually_exclusive_options=["databricks_token"],
        required_with_options=["databricks_client_id"],
        at_least_one_set=["databricks_client_id", "databricks_client_secret", "databricks_token"],
    ),
]

DATABRICKS_DATA_COLLECTOR_OPTIONS = [
    click.option(
        "--skip-secret-creation",
        help="Skip secret creation. Warning: Advanced use-case only.",
        default=False,
        show_default=True,
        is_flag=True,
    ),
    click.option(
        "--databricks-secret-key",
        help="Databricks secret key. This is a secret used by Databricks to communicate "
        "back to Monte Carlo.",
        default=DEFAULT_SECRET_NAME,
        show_default=True,
    ),
    click.option(
        "--databricks-secret-scope",
        help="Databricks secret scope. This is a secret used by Databricks to communicate "
        "back to Monte Carlo.",
        default=DEFAULT_SECRET_SCOPE,
        show_default=True,
    ),
    click.option(
        "--skip-notebook-creation",
        help="Skip notebook creation.",
        default=False,
        show_default=True,
        is_flag=True,
        cls=AdvancedOptions,
        required_with_options=[
            "databricks_job_id",
            "databricks_job_name",
            "databricks_notebook_path",
        ],
    ),
    click.option(
        "--databricks-job-id",
        help="Databricks job id, required if notebook creation is skipped.",
        required=False,
        cls=AdvancedOptions,
        required_with_options=["skip_notebook_creation"],
    ),
    click.option(
        "--databricks-job-name",
        help="Databricks job name, required if notebook creation is skipped.",
        required=False,
        cls=AdvancedOptions,
        required_with_options=["skip_notebook_creation"],
    ),
    click.option(
        "--databricks-notebook-path",
        help="Databricks notebook path, required if notebook creation is skipped.",
        required=False,
        cls=AdvancedOptions,
        required_with_options=["skip_notebook_creation"],
    ),
    click.option(
        "--databricks-notebook-source",
        help="Databricks notebook source, required if notebook creation is "
        'skipped. (e.g. "resources/databricks/notebook/v1/collection.py")',
        required=False,
        cls=AdvancedOptions,
        required_with_options=["skip_notebook_creation"],
    ),
]

S3_BUCKET_OPTIONS = [
    click.option(
        "--bucket",
        "bucket_name",
        required=False,
        help="Name of bucket to enable events for.",
    ),
    click.option(
        "--prefix",
        default=None,
        callback=convert_empty_str_callback,
        required=False,
        help="Limit the notifications to objects starting with a prefix (e.g. 'data/').",
    ),
    click.option(
        "--suffix",
        default=None,
        callback=convert_empty_str_callback,
        required=False,
        help="Limit notifications to objects ending with a suffix (e.g. '.csv').",
    ),
    click.option(
        "--topic-arn",
        default=None,
        callback=convert_empty_str_callback,
        required=False,
        help="Use an existing SNS topic (same region as the bucket). Creates a topic "
        "if one is not specified or "
        "if an MCD topic does not already exist in the region.",
    ),
    click.option(
        "--buckets-filename",
        required=False,
        help="Filename that contains bucket config to enable events for",
        cls=AdvancedOptions,
        mutually_exclusive_options=["bucket_name"],
    ),
]

EVENT_TYPE_OPTIONS = [
    click.option(
        "--event-type",
        required=True,
        default=CloudResourceService.MCD_EVENT_TYPE_MD,
        show_default=True,
        help="Type of event to setup.",
        type=click.Choice(
            list(CloudResourceService.MCD_EVENT_TYPE_FRIENDLY_TO_COLLECTOR_OUTPUT_MAP.keys()),
            case_sensitive=True,
        ),
    ),
]

RESOURCE_PROFILE_OPTIONS = [
    click.option(
        "--resource-aws-profile",
        "bucket_aws_profile",
        required=True,
        help="The AWS profile to use for operations on S3/SNS.",
    )
]

COLLECTOR_PROFILE_OPTIONS = [
    click.option(
        "--collector-aws-profile",
        required=True,
        help="The AWS profile to use for operations on SQS/Collector.",
    )
]

AUTO_YES_OPTIONS = [
    click.option(
        "--auto-yes",
        is_flag=True,
        help="Skip any interactive approval.",
        default=False,
        show_default=True,
    )
]

SELF_HOSTED_STREAMING_AUTH_OPTIONS = [
    click.option(
        "--auth-type",
        type=click.Choice(["NO_AUTH", "BASIC", "BEARER"]),
        required=True,
        help="The type of auth used to connect to the server",
    ),
    click.option(
        "--auth-token",
        required=False,
        help="The auth token used to connect to the server. Used for basic and bearer auth types.",
    ),
]


@click.group(help="Set up or manage an integration with Monte Carlo.")
def integrations():
    """
    Group for any integration related subcommands
    """
    pass


@integrations.command(help=f"Setup a Hive metastore integration (MySQL). {METADATA_VERBIAGE}.")
@click.pass_obj
@click.option("--port", help="HTTP port.", default=3306, type=click.INT, show_default=True)
@click.option(
    "--use-ssl",
    help="Use SSL to connect (using AWS RDS certificates).",
    required=False,
    is_flag=True,
    default=False,
    show_default=True,
)
@click.option(
    "--catalog",
    help="Presto catalog name. For using multiple hive clusters with Presto. "
    "Uses 'hive' if not specified.",
    required=False,
)
@add_common_options(warehouse_create_option())
@add_common_options(WAREHOUSE_OPTIONAL_CREATE_OPTIONS)
@add_common_options(DATABASE_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_hive_metastore(ctx, database, name, **kwargs):
    """
    Onboard a hive metastore connection (MySQL)
    """
    HiveOnboardingService(
        config=ctx["config"],
        command_name="integrations add_hive_metastore",
    ).onboard_hive_mysql(
        dbName=database,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a Presto SQL integration. {SQL_VERBIAGE}.")
@click.pass_obj
@click.option("--host", help="Hostname.", required=True)
@click.option("--port", help="HTTP port.", default=8889, type=click.INT, show_default=True)
@click.option("--user", help="Username with access to catalog/schema.", required=False)
@click.option(
    "--password",
    help=f"User's password. {PASSWORD_VERBIAGE}",
    required=False,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option("--catalog", help="Mount point to access data source.", required=False)
@click.option("--schema", help="Schema to access.", required=False)
@click.option(
    "--http-scheme",
    help="Scheme for authentication.",
    type=click.Choice(["http", "https"], case_sensitive=True),
    required=True,
)
@click.option(
    "--cert-file",
    help="Local SSL certificate file to upload to collector.",
    required=False,
    type=click.Path(dir_okay=False, exists=True),
    cls=AdvancedOptions,
    mutually_exclusive_options=["cert_s3"],
)
@click.option(
    "--aws-profile",
    help="AWS profile to be used when uploading cert file.",
    required=False,
    cls=AdvancedOptions,
    required_with_options=["cert_file"],
)
@click.option(
    "--aws-region",
    help="AWS region to be used when uploading cert file.",
    required=False,
    cls=AdvancedOptions,
    required_with_options=["cert_file"],
)
@click.option(
    "--cert-s3",
    help="Object path (key) to a certificate already uploaded to the collector.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["cert_file"],
)
@click.option(
    "--skip-cert-verification",
    help="Skip SSL certificate verification.",
    required=False,
    is_flag=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_presto(ctx, password, name, **kwargs):
    """
    Onboard a presto sql connection
    """
    if not password:
        password = None  # make explicitly null if not set. Prompts can't be None
    PrestoOnboardingService(
        config=ctx["config"],
        command_name="integrations add_presto",
    ).onboard_presto_sql(
        password=password,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a Hive SQL integration. {SQL_VERBIAGE}.")
@click.pass_obj
@click.option("--host", help="Hostname.", required=True)
@click.option("--database", help="Name of database.", required=False)
@click.option("--port", help="HTTP port.", default=10000, type=click.INT, show_default=True)
@click.option("--user", help="Username with access to hive.", required=True)
@click.option(
    "--auth-mode",
    help="Hive authentication mode.",
    required=False,
    default="SASL",
    type=click.Choice(["SASL", "NOSASL"]),
    show_default=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_hive(ctx, user, name, **kwargs):
    HiveOnboardingService(
        config=ctx["config"],
        command_name="integrations add_hive",
    ).onboard_hive_sql(
        username=user,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a Presto logs integration (S3). {QL_VERBIAGE}.")
@click.pass_obj
@add_common_options(S3_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_presto_logs(ctx, role, **kwargs):  # DEPRECATED
    """
    Onboard a presto logs (s3) connection
    """
    PrestoOnboardingService(
        config=ctx["config"],
        command_name="integrations add_presto_logs",
    ).onboard_presto_s3(assumable_role=role, **kwargs)


@integrations.command(help=f"Setup a Hive EMR logs integration (S3). {QL_VERBIAGE}.")
@click.pass_obj
@add_common_options(S3_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_hive_logs(ctx, role, **kwargs):  # DEPRECATED
    """
    Onboard a hive emr (s3) connection
    """
    HiveOnboardingService(
        config=ctx["config"],
        command_name="integrations add_hive_logs",
    ).onboard_hive_s3(assumable_role=role, **kwargs)


@integrations.command(help=f"Setup a Glue integration. {METADATA_VERBIAGE}.")
@click.pass_obj
@click.option("--region", help=f"Glue catalog region. {REGION_VERBIAGE}.", required=False)
@add_common_options(role_options())
@add_common_options(warehouse_create_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_glue(ctx, role, region, name, **kwargs):
    """
    Onboard a glue connection
    """
    GlueAthenaOnboardingService(
        config=ctx["config"],
        command_name="integrations add_glue",
    ).onboard_glue(
        assumable_role=role,
        aws_region=region,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help="Setup an Athena integration. For query logs and health queries.")
@click.pass_obj
@click.option(
    "--catalog",
    help="Glue data catalog. If not specified the AwsDataCatalog is used.",
    required=False,
)
@click.option(
    "--workgroup",
    help="Workbook for running queries and retrieving logs. If not specified the primary is used.",
    required=False,
)
@click.option("--region", help=f"Athena cluster region. {REGION_VERBIAGE}.", required=False)
@add_common_options(warehouse_select_option())
@add_common_options(role_options())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_athena(ctx, role, region, name, **kwargs):
    """
    Onboard an athena connection
    """
    GlueAthenaOnboardingService(
        config=ctx["config"],
        command_name="integrations add_athena",
    ).onboard_athena(
        assumable_role=role,
        aws_region=region,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a thrift binary Spark integration. {SQL_VERBIAGE}.")
@click.pass_obj
@click.option("--host", help="Hostname.", required=True)
@click.option("--database", help="Name of database.", required=True)
@click.option("--port", help="Port.", default=10000, type=click.INT, show_default=True)
@click.option("--user", help="Username with access to spark.", required=True)
@click.option(
    "--password",
    help=f"User's password. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_spark_binary_mode(ctx, user, name, **kwargs):
    """
    Onboard a spark connection, thrift binary mode
    """
    SparkOnboardingService(
        config=ctx["config"],
        command_name="integrations add_spark_binary_mode",
    ).onboard_spark(
        SPARK_BINARY_MODE_CONFIG_TYPE,
        username=user,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a thrift HTTP Spark integration. {SQL_VERBIAGE}.")
@click.pass_obj
@click.option("--url", help="HTTP URL.", required=True)
@click.option("--user", help="Username with access to spark.", required=True)
@click.option(
    "--password",
    help=f"User's password. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_spark_http_mode(ctx, user, name, **kwargs):
    """
    Onboard a spark connection, thrift http mode
    """
    SparkOnboardingService(
        config=ctx["config"],
        command_name="integrations add_spark_http_mode",
    ).onboard_spark(
        SPARK_HTTP_MODE_CONFIG_TYPE,
        username=user,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a Spark integration for Databricks. {SQL_VERBIAGE}.")
@click.pass_obj
@add_common_options(DATABRICKS_OPTIONS)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_spark_databricks(ctx, name, **kwargs):
    """
    Onboard a spark connection, databricks
    """
    SparkOnboardingService(
        config=ctx["config"],
        command_name="integrations add_spark_databricks",
    ).onboard_spark(
        SPARK_DATABRICKS_CONFIG_TYPE,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(
    help="Setup a Databricks metastore integration. For metadata and health queries."
)
@click.pass_obj
@add_common_options(warehouse_create_option())
@add_common_options(DATABRICKS_OPTIONS)
@add_common_options(DATABRICKS_DATA_COLLECTOR_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_databricks_metastore(ctx, name, **kwargs):
    """
    Onboard a databricks metastore
    """
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_databricks_metastore",
    ).onboard_databricks_metastore(
        connection_type=DATABRICKS_METASTORE_CONNECTION_TYPE,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(
    help="Setup a Databricks Delta integration. For metadata queries on delta tables "
    "when using an external metastore in databricks."
)
@click.pass_obj
@add_common_options(DATABRICKS_OPTIONS)
@add_common_options(warehouse_select_option())
@add_common_options(DATABRICKS_DATA_COLLECTOR_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_databricks_delta(ctx, name, **kwargs):
    """
    Onboard a databricks delta service
    """
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_databricks_delta",
    ).onboard_databricks_metastore(
        connection_type=DATABRICKS_DELTA_CONNECTION_TYPE, warehouseName=name, **kwargs
    )


@integrations.command(help=f"Setup a Databricks SQL Warehouse integration. {SQL_VERBIAGE}")
@click.pass_obj
@add_common_options(DATABRICKS_SQL_WAREHOUSE_OPTIONS)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_databricks_sql_warehouse(ctx, name, **kwargs):
    """
    Onboard a databricks sql warehouse connection
    """
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_databricks_sql_warehouse",
    ).onboard_databricks_sql_warehouse(
        connection_type=DATABRICKS_SQL_WAREHOUSE_CONNECTION_TYPE,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help="Setup a Databricks metastore sql warehouse integration. For metadata.")
@click.pass_obj
@add_common_options(warehouse_create_option())
@add_common_options(DATABRICKS_SQL_WAREHOUSE_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_databricks_metastore_sql_warehouse(ctx, name, **kwargs):
    """
    Onboard a databricks metastore sql warehouse
    """
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_databricks_metastore_sql_warehouse",
    ).onboard_databricks_sql_warehouse(
        connection_type=DATABRICKS_METASTORE_SQL_WAREHOUSE_CONNECTION_TYPE,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help="Create an integration key for a Databricks webhook")
@click.pass_obj
@click.option(
    "--integration-name",
    help="Name of associated Databricks metastore integration (required if you have more than one)",
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def create_databricks_webhook_key(ctx, integration_name: Optional[str], **kwargs):
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations create_databricks_webhook_key",
    ).create_webhook_key(
        warehouse_name=integration_name,
    )


@integrations.command(help=f"Setup a Redshift integration. {WAREHOUSE_VERBIAGE}.")
@click.pass_obj
@click.option("--port", help="HTTP port.", default=5439, type=click.INT, show_default=True)
@add_common_options(warehouse_create_option())
@add_common_options(DATABASE_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@add_common_options(NETWORK_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_redshift(ctx, database, name, **kwargs):
    """
    Onboard a redshift connection
    """
    WarehouseOnboardingService(
        config=ctx["config"],
        command_name="integrations add_redshift",
    ).onboard_redshift(
        dbName=database,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help=f"Setup a Snowflake integration. {WAREHOUSE_VERBIAGE}.")
@click.pass_obj
@click.option(
    "--user",
    help="User with access to snowflake.",
    required=True,
    cls=AdvancedOptions,
)
@click.option("--account", help="Snowflake account name.", required=True)
@click.option("--warehouse", help="Name of the warehouse for the user.", required=True)
@click.option(
    "--private-key",
    help="User's private key file for key pair auth.",
    type=click.Path(exists=True),
    required=True,
    cls=AdvancedOptions,
)
@click.option(
    "--private-key-passphrase",
    help="User's private key passphrase. This argument is "
    f"only needed when the private key is encrypted. {PASSWORD_VERBIAGE}.",
    required=False,
    cls=AdvancedOptions,
    prompt_if_requested=True,
    required_with_options=["private_key"],
)
@add_common_options(warehouse_create_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_snowflake(ctx, name, **kwargs):
    """
    Onboard a snowflake connection
    """
    WarehouseOnboardingService(
        config=ctx["config"],
        command_name="integrations add_snowflake",
    ).onboard_snowflake(warehouseName=name, **kwargs)


@integrations.command(help=f"Setup a BigQuery integration. {WAREHOUSE_VERBIAGE}.")
@click.pass_obj
@click.option("--key-file", help="JSON Key file.", type=click.Path(exists=True), required=True)
@add_common_options(warehouse_create_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_bigquery(ctx, key_file, name, **kwargs):
    """
    Onboard a BigQuery connection
    """
    WarehouseOnboardingService(
        config=ctx["config"],
        command_name="integrations add_bigquery",
    ).onboard_bq(ServiceFile=key_file, warehouseName=name, **kwargs)


@integrations.command(help="Setup an integration that uses self-hosted credentials.")
@click.pass_obj
@click.option(
    "--connection-type",
    help="Type of connection.",
    required=True,
    type=click.Choice(list(GQL_TO_FRIENDLY_CONNECTION_MAP.keys()), case_sensitive=False),
    cls=AdvancedOptions,
    values_with_required_options=CONNECTION_TO_WAREHOUSE_TYPE_MAP.keys(),
    required_options_for_values=["name"],
)
@click.option(
    "--mechanism",
    help="Credential self-hosting mechanism.",
    required=True,
    type=click.Choice(SELF_HOSTING_MECHANISMS, case_sensitive=False),
    default=SECRETS_MANAGER_CREDENTIAL_MECHANISM,
)
@click.option(
    "--key",
    help="Identifier for credentials within self-hosting mechanism.",
    required=True,
)
@click.option("--name", help="Friendly name for the warehouse.", required=False)
@add_common_options(role_options(required=False))
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_self_hosted_credentials(ctx, mechanism, key, role, name, **kwargs):
    """
    Onboard a connection with self-hosted credentials
    """
    SelfHostedCredentialOnboardingService(
        config=ctx["config"],
        command_name="integrations add_self_hosted_credentials",
    ).onboard_connection(
        self_hosting_mechanism=mechanism,
        self_hosting_key=key,
        assumable_role=role,
        warehouseName=name,
        **kwargs,
    )


@integrations.command(help="Setup an Airflow integration to receive events from Airflow callbacks.")
@click.pass_obj
@click.option("--host", help="Hostname.", required=True)
@add_common_options(etl_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_airflow(ctx, host, name, **kwargs):
    """
    Onboard an airflow connection
    """
    etl_name = name or "airflow"
    AirflowOnboardingService(
        config=ctx["config"],
        command_name="integrations add_airflow",
    ).onboard_airflow(
        etl_name=etl_name,
        hostName=host,
        **kwargs,
    )


@integrations.command(help=f"Configure S3 metadata events. {EVENT_VERBIAGE}.")
@click.pass_obj
@click.option(
    "--connection-type",
    help="Type of the integration.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["connection_id"],
    type=click.Choice(
        [
            DATABRICKS_METASTORE_CONNECTION_TYPE,
            GLUE_CONNECTION_TYPE,
            HIVE_MYSQL_CONNECTION_TYPE,
        ],
        case_sensitive=False,
    ),
    required=True,
)
@add_common_options(warehouse_create_option(default="s3-metadata-events"))
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def configure_metadata_events(ctx, **kwargs):
    """
    Configure S3 metadata events for a lake
    """
    EventsOnboardingService(
        config=ctx["config"],
        command_name="integrations configure_metadata_events",
    ).configure_metadata_events(**kwargs)


@integrations.command(help=f"Configure S3 query log events. {EVENT_VERBIAGE}.")
@click.pass_obj
@click.option(
    "--connection-type",
    help="Type of the integration.",
    type=click.Choice([HIVE_S3_CONNECTION_TYPE, PRESTO_S3_CONNECTION_TYPE], case_sensitive=True),
    required=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(role_options())
@click.option(
    "--format-type",
    help="Query log format.",
    type=click.Choice(["hive-emr", "hive-native", "custom"], case_sensitive=True),
    required=True,
)
@click.option(
    "--source-format",
    help='Query log file format. Only required when "custom" is used.',
    type=click.Choice(["json", "jsonl"], case_sensitive=True),
    required=False,
)
@click.option(
    "--mapping-file",
    help='Mapping of expected to existing query log fields. Only required if "custom" is used.',
    type=click.Path(exists=True),
    required=False,
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def configure_query_log_events(ctx, **kwargs):
    """
    Configure S3 query log events for a lake
    """
    EventsOnboardingService(
        config=ctx["config"],
        command_name="integrations configure_query_log_events",
    ).configure_query_log_events(**kwargs)


@integrations.command(help="Disable S3 metadata events.")
@click.pass_obj
@click.option("--name", help="Resource name (only required if more than one exists)")
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def disable_metadata_events(ctx, **kwargs):
    """
    Configure S3 metadata events
    """
    EventsOnboardingService(
        config=ctx["config"],
        command_name="integrations disable_metadata_events",
    ).disable_metadata_events(**kwargs)


@integrations.command(help="Disable S3 query log events.")
@click.pass_obj
@click.option("--name", help="Resource name (only required if more than one exists)")
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def disable_query_log_events(ctx, **kwargs):
    """
    Configure S3 query log events
    """
    EventsOnboardingService(
        config=ctx["config"],
        command_name="integrations disable_query_log_events",
    ).disable_query_log_events(**kwargs)


@integrations.command(help=f"Setup a Tableau integration. {BI_VERBIAGE}.")
@click.pass_obj
@click.option(
    "--token-name",
    help="Name for the personal access token.",
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "password",
        "user",
        "username",
        "client_id",
        "secret_id",
        "secret_value",
    ],
    required_with_options=["token_value"],
    at_least_one_set=[
        "password",
        "user",
        "username",
        "token_name",
        "token_value",
        "client_id",
        "secret_id",
        "secret_value",
    ],
)
@click.option(
    "--token-value",
    help=f"Value for the personal access token. {PASSWORD_VERBIAGE}.",
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "password",
        "user",
        "username",
        "client_id",
        "secret_id",
        "secret_value",
    ],
    required_with_options=["token_name"],
    prompt_if_requested=True,
)
@click.option(
    "--password",
    help=f"Password for the service account. {PASSWORD_VERBIAGE}.",
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "token_name",
        "token_value",
        "username",
        "client_id",
        "secret_id",
        "secret_value",
    ],
    required_with_options=["user"],
    prompt_if_requested=True,
)
@click.option(
    "--user",
    help="Username for the service account.",
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "token_name",
        "token_value",
        "username",
        "client_id",
        "secret_id",
        "secret_value",
    ],
    required_with_options=["password"],
)
@click.option(
    "--username",
    help="Username for the Connected App.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["token_name", "token_value", "user", "password"],
    required_with_options=["client_id", "secret_id", "secret_value"],
)
@click.option(
    "--client-id",
    help="Client ID for the Connected App.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["token_name", "token_value", "user", "password"],
    required_with_options=["username", "secret_id", "secret_value"],
)
@click.option(
    "--secret-id",
    help="Secret ID for the Connected App.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["token_name", "token_value", "user", "password"],
    required_with_options=["username", "client_id", "secret_value"],
)
@click.option(
    "--secret-value",
    help=f"Value of the Connected App secret. {PASSWORD_VERBIAGE}",
    cls=AdvancedOptions,
    mutually_exclusive_options=["token_name", "token_value", "user", "password"],
    required_with_options=["username", "client_id", "secret_id"],
    prompt_if_requested=True,
)
@click.option("--site-name", help="The Tableau site name.", required=True)
@click.option("--server-name", help="The Tableau server name.", required=True)
@click.option("--name", help="Friendly name for the Tableau integration.", required=False)
@add_common_options(BI_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_tableau(ctx, user, username, **kwargs):
    """
    Onboard a tableau connection
    """
    ReportsOnboardingService(
        config=ctx["config"],
        command_name="integrations add_tableau",
    ).onboard_tableau(
        username=user or username,
        **kwargs,
    )


@integrations.command(help=f"Setup a Looker metadata integration. {BI_VERBIAGE}.")
@click.pass_obj
@click.option("--host-url", "base_url", help="Looker host url.", required=True)
@click.option("--client-id", help="Looker client id.", required=True)
@click.option(
    "--client-secret",
    help=f"Looker client secret (API key). {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option("--name", help="Friendly name for the Looker integration.", required=False)
@add_common_options(BI_OPTIONS)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_looker(ctx, **kwargs):
    """
    Onboard a looker metadata connection
    """
    ReportsOnboardingService(
        config=ctx["config"],
        command_name="integrations add_looker",
    ).onboard_looker_metadata(**kwargs)


@integrations.command(
    help="""
Create or refresh details of BI warehouse sources. Warehouse sources are warehouses connected to a BI container.

If only the bi-container-id parameter is supplied, then the behavior will be the following:
\b
\n
(1) All warehouse sources in the customer's BI system will be matched against warehouses in Monte Carlo.
\b
\n
(2) If all of the customer's BI warehouse sources have a match in Monte Carlo, then the warehouse source details will be
saved.
\b
\n
(3) Otherwise, the response will contain the details of customer's BI warehouse sources and Monte Carlo warehouses. The
caller will then need to manually save the warehouse source details by specifying the warehouse-source-details
parameter.
\b
\n
If the warehouse-source-details parameter is also supplied, then the warehouses will be validated and saved.
"""  # noqa
)
@click.pass_obj
@click.option(
    "--bi-container-id",
    help="ID of the BI container for which to refresh warehouse sources.",
    required=True,
)
@click.option(
    "--warehouse-source-details",
    help="""
              An optional JSON array of warehouse sources. If supplied, these warehouse sources will be created. If an
              entry with the same bi_container_id, warehouse_resource_id and warehouse_resource_type is found, its
              bi_warehouse_id will be updated.

              Each JSON object in this array has the following fields:
              \b
              \n
              bi_warehouse_id: The ID of the warehouse in the customer's ID space.
              \b
              \n
              warehouse_resource_id: The ID of the warehouse Monte Carlo's ID space.
              \b
              \n
              warehouse_resource_type: The type of the warehouse.
              \b
              \n
              E.g. --warehouse-source-details '[{"bi_warehouse_id":"0dd5d40e-8749-71cb-9fa5-2e33b570ff43",
              "warehouse_resource_id": "11b1ad6f-e35c-4532-84d8-2fc88bd53660", "warehouse_resource_type": "snowflake"}]'
              """,  # noqa
    required=False,
    callback=validate_json_callback,
)
def refresh_bi_to_warehouse_connections(ctx, bi_container_id, warehouse_source_details, **kwargs):
    BiService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations refresh_bi_to_warehouse_connections",
    ).refresh_bi_to_warehouse_connections(bi_container_id, warehouse_source_details)


@integrations.command(help=f"Setup a Looker ML (git) integration. {BI_VERBIAGE}.")
@click.pass_obj
@click.option(
    "--ssh-key",
    help="The ssh key for git ssh integrations.",
    required=False,
    type=click.Path(dir_okay=False, exists=True),
    cls=AdvancedOptions,
    mutually_exclusive_options=["token", "username"],
)
@click.option(
    "--repo-url",
    help="Repository URL as ssh://[user@]server/project.git or the shorter "
    "form [user@]server:project.git for ssh. For https, use https://server/project.git.",
    required=True,
)
@click.option(
    "--token",
    help="Git Access Token to be used for Https instead of ssh key.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["ssh-key"],
)
@click.option(
    "--username",
    help="Git username to be used in conjunction with the access token. This is only "
    "required for BitBucket integrations.",
    cls=AdvancedOptions,
    required=False,
    mutually_exclusive_options=["ssh-key"],
)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_looker_git(ctx, **kwargs):
    """
    Onboard a looker metadata connection
    """
    ReportsOnboardingService(
        config=ctx["config"],
        command_name="integrations add_looker_git",
    ).onboard_looker_git(**kwargs)


@integrations.command(help=f"Setup a Power BI integration. {BI_VERBIAGE}.")
@click.pass_obj
@click.option("--tenant-id", help="The tenant ID from the Azure Active Directory.", required=True)
@click.option(
    "--auth-mode",
    help="Authentication Mode. We support service principal and primary user two auth types",
    required=True,
    type=click.Choice(["SERVICE_PRINCIPAL", "PRIMARY_USER"]),
)
@click.option(
    "--client-id",
    help="App registration application ID for accessing Power BI.",
    required=True,
    cls=AdvancedOptions,
)
@click.option(
    "--client-secret",
    help="Secret for the application to access the Power BI. Set only when auth-mode is "
    "SERVICE_PRINCIPAL",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["username", "password"],
    prompt_if_requested=True,
)
@click.option(
    "--username",
    help="Username for accessing the Power BI. Set only when auth-mode is PRIMARY_USER.",
    cls=AdvancedOptions,
    required=False,
    mutually_exclusive_options=["client-secret"],
)
@click.option(
    "--password",
    help="Password for accessing the Power BI. Set only when auth-mode is PRIMARY_USER.",
    cls=AdvancedOptions,
    required=False,
    mutually_exclusive_options=["client-secret"],
    prompt_if_requested=True,
)
@click.option("--name", help="Friendly name for the Power BI integration.", required=False)
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_power_bi(ctx, **kwargs):
    """
    Onboard a Power BI connection
    """
    ReportsOnboardingService(
        config=ctx["config"],
        command_name="integrations add_power_bi",
    ).onboard_power_bi(**kwargs)


@integrations.command(help="Setup a dbt Cloud integration.")
@click.pass_obj
@click.option(
    "--dbt-cloud-api-token",
    "dbt_cloud_api_token",
    help=f"dbt Cloud API token. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option(
    "--dbt-cloud-account-id",
    "dbt_cloud_account_id",
    help="dbt Cloud Account ID.",
    required=True,
)
@click.option(
    "--dbt-cloud-base-url",
    "dbt_cloud_base_url",
    help="dbt Cloud Base URL.",
    required=False,
)
@click.option(
    "--webhook-hmac-secret",
    "webhook_hmac_secret",
    help="The secret token `hmac_secret` provided by dbt after a webhook "
    "is successfully created in dbt.",
    required=True,
)
@click.option(
    "--webhook-id",
    "webhook_id",
    help="The webhook id provided by dbt after a webhook is successfully created in dbt.",
    required=True,
)
@add_common_options(warehouse_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_dbt_cloud(ctx, name, **kwargs):
    """
    Onboard a dbt cloud connection
    """
    DbtCloudOnboardingService(
        config=ctx["config"],
        command_name="integrations add_dbt_cloud",
    ).onboard_dbt_cloud(warehouseName=name, **kwargs)


@integrations.command(help="Setup a Fivetran integration.")
@click.pass_obj
@click.option(
    "--fivetran-api-key",
    "fivetran_api_key",
    help=f"Fivetran API Key. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option(
    "--fivetran-api-password",
    "fivetran_api_password",
    help=f"Fivetran API Key. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option(
    "--fivetran-base-url",
    "fivetran_base_url",
    help="Fivetran Base URL.",
    required=False,
)
@add_common_options(etl_select_option())
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_fivetran(ctx, name, **kwargs):
    """
    Onboard a Fivetran connection
    """
    FivetranOnboardingService(
        config=ctx["config"],
        command_name="integrations add_fivetran",
    ).onboard_fivetran(etl_name=name, **kwargs)


@integrations.command(help="Setup an Azure Data Factory integration.")
@click.pass_obj
@click.option(
    "--tenant-id",
    help="Azure Tenant ID.",
    required=True,
)
@click.option(
    "--client-id",
    help="Azure Client ID.",
    required=True,
)
@click.option(
    "--client-secret",
    help=f"Azure Client Secret. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option(
    "--subscription-id",
    help="Azure Subscription ID.",
    required=True,
)
@click.option(
    "--resource-group-name",
    help="Azure Resource Group Name.",
    required=True,
)
@click.option(
    "--factory-name",
    help="Azure Data Factory Name.",
    required=True,
)
@add_common_options(etl_select_option(required=True))
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_azure_data_factory(ctx, **kwargs):
    """
    Onboard an Azure Data Factory connection
    """
    AzureDataFactoryOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_azure_data_factory",
    ).onboard(**kwargs)


@integrations.command(help="Setup an Informatica integration.")
@click.pass_obj
@click.option(
    "--username",
    help=f"Informatica username. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@click.option(
    "--password",
    help=f"Informatica password. {PASSWORD_VERBIAGE}.",
    required=True,
    cls=AdvancedOptions,
    prompt_if_requested=True,
)
@add_common_options(etl_select_option(required=True))
@add_common_options(ONBOARDING_CONFIGURATION_OPTIONS)
def add_informatica(ctx, **kwargs):
    """
    Onboard an Informatica connection
    """
    InformaticaOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_informatica",
    ).onboard(**kwargs)


@integrations.command(help=f"Setup a SQL Server integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=1433)
    + BASIC_DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_sql_server(ctx, name, **kwargs):
    """
    Onboard a SQL Server connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_sql_server",
    ).onboard_transactional_db(warehouseName=name, dbType=SQL_SERVER_DB_TYPE, **kwargs)


@integrations.command(help=f"Setup a Postgres integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=5432)
    + DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_postgres(ctx, database, name, **kwargs):
    """
    Onboard a Postgres connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_postgres",
    ).onboard_transactional_db(
        warehouseName=name, dbName=database, dbType=POSTGRES_DB_TYPE, **kwargs
    )


@integrations.command(help=f"Setup a MySQL integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=3306)
    + BASIC_DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
    + SSL_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_mysql(ctx, name, **kwargs):
    """
    Onboard a MySQL connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_mysql",
    ).onboard_transactional_db(warehouseName=name, dbType=MYSQL_DB_TYPE, **kwargs)


@integrations.command(help=f"Setup an Oracle integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=1521)
    + DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_oracle(ctx, database, name, **kwargs):
    """
    Onboard an Oracle connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_oracle",
    ).onboard_transactional_db(warehouseName=name, dbName=database, dbType=ORACLE_DB_TYPE, **kwargs)


@integrations.command(help=f"Setup an MariaDB integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=3306)
    + BASIC_DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_mariadb(ctx, name, **kwargs):
    """
    Onboard an MariaDB connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_mariadb",
    ).onboard_transactional_db(warehouseName=name, dbType=MARIADB_DB_TYPE, **kwargs)


@integrations.command(help=f"Setup a Teradata integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=1025)
    + BASIC_DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click.option(
    "--sslmode",
    type=click.Choice(["ALLOW", "PREFER", "REQUIRE", "VERIFY-CA", "VERIFY-FULL"]),
    default="PREFER",
    show_default=True,
    help="SSL mode for connections to Teradata.",
    required=False,
)
@click.option(
    "--logmech",
    type=click.Choice(["TD2", "BROWSER", "JWT", "LDAP", "KRB5", "TDNEGO"]),
    default="TD2",
    show_default=True,
    help="Logon mechanism for Teradata connection.",
    required=False,
)
def add_teradata(ctx, name, sslmode, logmech, **kwargs):
    """
    Onboard a Teradata connection
    """
    kwargs["connection_settings"] = {
        key: value
        for key, value in [("td_sslmode", sslmode), ("td_logmech", logmech)]
        if value is not None
    }

    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_teradata",
    ).onboard_transactional_db(warehouseName=name, dbType=TERADATA_DB_TYPE, **kwargs)


@integrations.command(
    help=f"Setup an Azure Dedicated SQL Pool integration. {LIGHTWEIGHT_VERBIAGE}."
)
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=1433)
    + DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_azure_dedicated_sql_pool(ctx, database, name, **kwargs):
    """
    Onboard an Azure Dedicated SQL Pool connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_azure_dedicated_sql_pool",
    ).onboard_transactional_db(
        warehouseName=name,
        dbName=database,
        dbType=AZURE_DEDICATED_SQL_POOL_TYPE,
        **kwargs,
    )


@integrations.command(help=f"Setup an Azure SQL Database integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=1433)
    + DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_azure_sql_database(ctx, database, name, **kwargs):
    """
    Onboard an Azure SQL Database connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_azure_sql_database",
    ).onboard_transactional_db(
        warehouseName=name,
        dbName=database,
        dbType=AZURE_SQL_DATABASE_TYPE,
        **kwargs,
    )


@integrations.command(help=f"Setup a SAP HANA integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + port_create_option(default_port=39015)
    + DATABASE_OPTIONS
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_sap_hana_database(ctx, database, name, **kwargs):
    """
    Onboard a SAP HANA Database connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_sap_hana_database",
    ).onboard_transactional_db(
        warehouseName=name,
        dbName=database,
        dbType=SAP_HANA_DATABASE_TYPE,
        **kwargs,
    )


@integrations.command(help=f"Setup a Motherduck integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + [
        click.option("--token", help="Token for authentication", required=True),
        click.option("--database", help="Name of default database.", required=True),
    ]
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_motherduck_database(ctx, database, name, **kwargs):
    """
    Onboard a Motherduck connection
    """
    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_motherduck_database",
    ).onboard_transactional_db(
        warehouseName=name,
        dbName=database,
        dbType=MOTHERDUCK_DATABASE_TYPE,
        **kwargs,
    )


@integrations.command(help=f"Setup a Dremio integration. {LIGHTWEIGHT_VERBIAGE}.")
@click.pass_obj
@add_common_options(
    warehouse_create_option(required=True)
    + [
        click.option("--token", help="Token for authentication", required=True),
        click.option(
            "--host",
            help="Hostname of coordinator node or data.dremio.cloud if using Dremio cloud.",
            required=True,
        ),
        click.option(
            "--port",
            help="Dremio's Arrow Flight server port. 443 if using Dremio Cloud",
            required=True,
        ),
    ]
    + ONBOARDING_CONFIGURATION_OPTIONS
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
@click.option(
    "--tls",
    is_flag=True,
    help="Use TLS for connection. Required for Dremio cloud.",
    required=False,
)
def add_dremio(ctx, name, tls, **kwargs):
    """
    Onboard a Dremio connection
    """
    if tls:
        kwargs["connection_settings"] = {"tls": True}

    TransactionalOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_dremio",
    ).onboard_transactional_db(
        warehouseName=name,
        dbType=DREMIO_DATABASE_TYPE,
        **kwargs,
    )


@integrations.command(help="List all active connections.", name="list")
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def display_integrations(ctx):
    """
    Display active integrations
    """
    OnboardingStatusService(
        config=ctx["config"],
        command_name="integrations display_integrations",
    ).display_integrations()


@integrations.command(
    help="Create an IAM role from a policy FILE. "
    "The returned role ARN and external id should be used for adding lake assets."
)
@click.pass_obj
@click.argument("file", type=click.Path(dir_okay=False, exists=True))
@click.option(
    "--aws-profile",
    required=True,
    help="The AWS profile indicating where the role is created.",
)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
def create_role(ctx, file, aws_profile, dc_id, agent_id):
    """
    Create a collector compatible role from the provided policy
    """
    CloudResourceService(
        config=ctx["config"],
        aws_profile_override=aws_profile,
        command_name="integrations create_role",
    ).create_role(path_to_policy_doc=file, dc_id=dc_id, agent_id=agent_id)


@integrations.command(
    help="Update credentials for a connection. Only replaces/inserts the "
    "keys in changes by default.",
    name="update",
)
@click.pass_obj
@add_common_options(CONNECTION_OPTIONS)
@click.option(
    "--changes",
    help="""
              Credential key,value pairs as JSON.
              \b
              \n
              E.g. --changes '{"user":"Apollo"}'
              """,
    required=True,
    callback=validate_json_callback,
)
@click.option(
    "--skip-validation",
    help="Skip validating credentials.",
    default=False,
    show_default=True,
    is_flag=True,
)
@click.option(
    "--replace-all",
    help="Replace all credentials rather than just inserting/updating the keys in changes.",
    default=False,
    show_default=True,
    is_flag=True,
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def update_credentials(ctx, connection_id, changes, skip_validation, replace_all):
    """
    Update credentials for a connection
    """
    ConnectionOperationsService(
        config=ctx["config"],
        command_name="integrations update_credentials",
    ).update_credentials(
        connection_id=connection_id,
        changes=changes,
        should_validate=not skip_validation,
        should_replace=replace_all,
    )


@integrations.command(
    help="Remove an existing connection. Deletes any associated jobs, monitors, etc.",
    name="remove",
)
@click.pass_obj
@add_common_options(CONNECTION_OPTIONS)
@click.option(
    "--no-prompt",
    help="Don't ask for confirmation.",
    default=False,
    show_default=True,
    is_flag=True,
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def remove_connection(ctx, **kwargs):
    """
    Remove connection by ID
    """
    ConnectionOperationsService(
        config=ctx["config"],
        command_name="integrations remove_connection",
    ).remove_connection(**kwargs)


@integrations.command(help="Retest an existing connection.", name="test")
@click.pass_obj
@add_common_options(CONNECTION_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def echo_test_existing(ctx, **kwargs):
    """
    Tests an existing connection and echos results in pretty JSON.
    """
    ConnectionOperationsService(
        config=ctx["config"],
        command_name="integrations echo_test_existing",
    ).echo_test_existing(**kwargs)


@integrations.command(help="Setup complete S3 event notifications for a lake.")
@click.pass_obj
@add_common_options(S3_BUCKET_OPTIONS)
@add_common_options(EVENT_TYPE_OPTIONS)
@add_common_options(COLLECTOR_PROFILE_OPTIONS)
@add_common_options(RESOURCE_PROFILE_OPTIONS)
@add_common_options(AUTO_YES_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_events(ctx, **kwargs):
    CloudResourceService(
        config=ctx["config"],
        aws_wrapper=None,
        command_name="integrations add_events",
    ).add_events(**kwargs)


@integrations.command(help="Setup Event Topic for S3 event notifications in a lake.")
@click.pass_obj
@add_common_options(S3_BUCKET_OPTIONS)
@add_common_options(EVENT_TYPE_OPTIONS)
@add_common_options(RESOURCE_PROFILE_OPTIONS)
@add_common_options(AUTO_YES_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def create_event_topic(ctx, **kwargs):
    CloudResourceService(
        config=ctx["config"],
        aws_wrapper=None,
        command_name="integrations create_event_topic",
    ).create_event_topic(**kwargs)


@integrations.command(help="Setup Bucket Side S3 event infrastructure for a lake.")
@click.pass_obj
@add_common_options(S3_BUCKET_OPTIONS)
@add_common_options(EVENT_TYPE_OPTIONS)
@add_common_options(RESOURCE_PROFILE_OPTIONS)
@add_common_options(AUTO_YES_OPTIONS)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def create_bucket_side_event_infrastructure(ctx, **kwargs):
    CloudResourceService(
        config=ctx["config"],
        aws_wrapper=None,
        command_name="integrations create_bucket_side_event_infrastructure",
    ).create_bucket_side_event_infrastructure(**kwargs)


@integrations.command(
    help="Create an integration key. The resulting key id and secret will "
    "be printed to the console."
)
@click.pass_obj
@click.option("--description", required=True, help="Key description.")
@click.option(
    "--scope",
    required=True,
    type=click.Choice(IntegrationKeyScope.values(), case_sensitive=False),
    help="Key scope (integration the key can be used for).",
)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def create_key(ctx, **kwargs):
    IntegrationKeyService(
        config=ctx["config"],
        command_name="integrations create_key",
    ).create(**kwargs)


@integrations.command(help="Delete an integration key.")
@click.pass_obj
@click.option("--key-id", required=True, help="Integration key id.")
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def delete_key(ctx, **kwargs):
    IntegrationKeyService(
        config=ctx["config"],
        command_name="integrations delete_key",
    ).delete(**kwargs)


@integrations.command(help="List all integration keys.")
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def list_keys(ctx):
    IntegrationKeyService(
        config=ctx["config"],
        command_name="integrations list_keys",
    ).get_all()


@integrations.command(help="Set the name of a Warehouse.")
@click.pass_obj
@click.option("--current-name", required=True, help="Current Name of the Warehouse")
@click.option("--new-name", required=True, help="Name to give the Warehouse")
def set_warehouse_name(ctx, **kwargs):
    ConnectionOperationsService(
        config=ctx["config"],
        command_name="integrations set_warehouse_name",
    ).set_warehouse_name(**kwargs)


@integrations.command(help="Set the name of a BI Connection.")
@click.pass_obj
@click.option("--bi-connection-id", required=True, help="UUID of the existing BI Connection")
@click.option("--new-name", required=True, help="Name to give the BI Connection")
def set_bi_connection_name(ctx, **kwargs):
    ConnectionOperationsService(
        config=ctx["config"],
        command_name="integrations set_bi_connection_name",
    ).set_bi_connection_name(**kwargs)


@integrations.command(help="Update Databricks Notebook to the latest version.")
@click.pass_obj
@add_common_options(CONNECTION_OPTIONS)
def update_databricks_notebook(ctx, **kwargs):
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations update_databricks_notebook",
    ).update_databricks_notebook(**kwargs)


@integrations.command(help="Get the Databricks job info for your connection.")
@click.pass_obj
@add_common_options(CONNECTION_OPTIONS)
def show_databricks_metadata_job_info(ctx, **kwargs):
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations show_databricks_metadata_job_info",
    ).get_databricks_job_info(**kwargs)


@integrations.command(help="Get the most up to date databricks notebook version.")
@click.pass_obj
def show_current_notebook_version(ctx):
    DatabricksOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations show_current_notebook_version",
    ).get_current_databricks_notebook_version()


@integrations.command(help="Setup a streaming system.")
@click.pass_obj
@click.option(
    "--streaming-system-type",
    required=True,
    type=click.Choice(["confluent-cloud", "msk", "self-hosted"]),
    help="Streaming System type. Currently we only support confluent-cloud and self-hosted.",
)
@click.option(
    "--streaming-system-name",
    required=True,
    help="Name that helps you identify the streaming system in MC.",
)
@click.option(
    "--dc-id",
    help="The data collector UUID that you'd like to run jobs of this system in. "
    "This is needed when you have more than one active data collector.",
    required=False,
    cls=AdvancedOptions,
)
def add_streaming_system(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_streaming_system",
    ).create_streaming_system(**kwargs)


@integrations.command(help="Test and generate credential key for Confluent Kafka Connection.")
@click.pass_obj
@click.option(
    "--cluster",
    required=True,
    help="Cluster ID in Confluent Cloud.",
)
@click.option(
    "--api-key",
    required=True,
    help="API Key.",
)
@click.option(
    "--secret",
    help="Secret of the API Key.",
    required=True,
)
@click.option(
    "--url",
    help="URL for accessing the Kafka Cluster in Confluent Cloud.",
    required=True,
)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the "
    "specific dc, or you have multiple DC.",
    required=False,
)
def test_confluent_kafka_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_confluent_kafka_credentials",
    ).test_new_confluent_kafka_credentials(**kwargs)


@integrations.command(
    help="Test and generate credential key for Confluent Cloud accessing Kafka Connect."
)
@click.pass_obj
@click.option(
    "--confluent-env",
    required=True,
    help="Environment ID in Confluent Cloud.",
)
@click.option(
    "--cluster",
    required=True,
    help="Cluster ID of the Kafka Connect in Confluent Cloud.",
)
@click.option(
    "--api-key",
    required=True,
    help="API Key.",
)
@click.option(
    "--secret",
    help="Secret of the API Key.",
    required=True,
)
@click.option(
    "--url",
    help="Special URL for accessing Kafka Connect API in Confluent Cloud. By default, "
    "we use the cloud API URL.",
    required=False,
)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the specific dc, "
    "or you have multiple DC.",
    required=False,
)
def test_confluent_kafka_connect_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_confluent_kafka_connect_credentials",
    ).test_new_confluent_kafka_connect_credentials(**kwargs)


@integrations.command(
    help="Test and generate credential key for a MSK Kafka Connection (via REST proxy)."
)
@click.pass_obj
@click.option(
    "--cluster",
    required=True,
    help="Cluster ID",
)
@click.option(
    "--url",
    help="URL for accessing the REST Proxy.",
    required=True,
)
@add_common_options(SELF_HOSTED_STREAMING_AUTH_OPTIONS)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the specific dc, "
    "or you have multiple DC.",
    required=False,
)
def test_msk_kafka_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_msk_kafka_credentials",
    ).test_new_msk_kafka_credentials(**kwargs)


@integrations.command(help="Test and generate credential key for a MSK Kafka Connect Connection.")
@click.pass_obj
@click.option(
    "--cluster-arn",
    help="ARN of the MSK cluster the connectors are running against.",
    required=True,
)
@click.option(
    "--iam-role-arn",
    help="ARN of an assumable IAM role that will be used for collection.",
    required=True,
)
@click.option(
    "--external-id",
    help="Optional external id, if required to assume IAM role for collection.",
    required=True,
)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the specific dc, "
    "or you have multiple DC.",
    required=False,
)
def test_msk_kafka_connect_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_msk_kafka_connect_credentials",
    ).test_new_msk_kafka_connect_credentials(**kwargs)


@integrations.command(help="Test and generate credential key for a Self Hosted Kafka Connection.")
@click.pass_obj
@click.option(
    "--cluster",
    required=True,
    help="Cluster ID",
)
@click.option(
    "--url",
    help="URL for accessing the self hosted Kafka Rest Proxy.",
    required=True,
)
@add_common_options(SELF_HOSTED_STREAMING_AUTH_OPTIONS)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the specific dc, "
    "or you have multiple DC.",
    required=False,
)
def test_self_hosted_kafka_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_self_hosted_kafka_credentials",
    ).test_new_self_hosted_kafka_credentials(**kwargs)


@integrations.command(help="Test and generate credential key for a Self Hosted Kafka Connect.")
@click.pass_obj
@click.option(
    "--cluster",
    required=True,
    help="Cluster ID of the Kafka Connect.",
)
@click.option(
    "--url",
    help="URL for accessing the self hosted Kafka Connect Rest Server.",
    required=True,
)
@add_common_options(SELF_HOSTED_STREAMING_AUTH_OPTIONS)
@click.option(
    "--dc-id",
    help="Data Collector UUID, if we'd like to test the credentials against the specific dc, "
    "or you have multiple DC.",
    required=False,
)
def test_self_hosted_kafka_connect_credentials(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations test_self_hosted_kafka_connect_credentials",
    ).test_new_self_hosted_kafka_connect_credentials(**kwargs)


@integrations.command(help=".")
@click.pass_obj
@click.option(
    "--connection-type",
    help="Streaming Connection Type to create. Right now only support "
    "['confluent-kafka', 'confluent-kafka-connect', "
    "'msk-kafka', 'msk-kafka-connect', 'self-hosted-kafka', 'self-hosted-kafka-connect']",
    type=click.Choice(
        [
            "confluent-kafka",
            "confluent-kafka-connect",
            "msk-kafka",
            "msk-kafka-connect",
            "self-hosted-kafka",
            "self-hosted-kafka-connect",
        ]
    ),
    required=True,
)
@click.option(
    "--key",
    help="Credentials key."
    "You can obtain it by calling test-confluent-kafka-credentials for connection type "
    "confluent-kafka; Or calling test-confluent-kafka-connect-credentials for connection type "
    "confluent-kafka-connect. It's ok to leave it blank, but fill [url, api-key, secret] field "
    "for confluent-kafka connection in this all; Or fill [api-key, secret, confluent-env] field "
    "for confluent-kafka-connect connection.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["url", "confluent_env", "api_key", "secret"],
)
@click.option(
    "--url",
    help="This is required only when the key is not specified and the connection "
    "type is confluent-kafka.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
)
@click.option(
    "--confluent-env",
    help="This is required only when the key is not specified and the connection "
    "type is confluent-kafka-connect.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
)
@click.option(
    "--api-key",
    help="API for accessing the  This is required only when the credential key is not specified.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
)
@click.option(
    "--secret",
    help="This is required only when the key is not specified.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
    prompt_if_requested=True,
)
@click.option(
    "--auth-type",
    type=click.Choice(["NO_AUTH", "BASIC", "BEARER"]),
    required=False,
    help="This is required only when the key is not specified and the cluster is self-hosted.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
)
@click.option(
    "--auth-token",
    required=False,
    help="This is required only when the key is not specified and the cluster is self-hosted.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
    prompt_if_requested=True,
)
@click.option(
    "--cluster-arn",
    required=False,
    help="This is required only when the key is not specified and the cluster type is MSK Connect.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
    prompt_if_requested=True,
)
@click.option(
    "--iam-role-arn",
    required=False,
    help="This is required only when the key is not specified and the cluster type is MSK Connect.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
    prompt_if_requested=True,
)
@click.option(
    "--external-id",
    required=False,
    help="This is required only when the key is not specified and the cluster type is MSK Connect.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["key"],
    prompt_if_requested=True,
)
@click.option(
    "--streaming-system-id",
    help=(
        "Streaming system UUID. If we are adding a cluster to an existing streaming system, we "
        "should use the UUID here. When this is given, please leave new_streaming_system_name, "
        "new_streaming_system_type, dc_id empty."
    ),
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "new_streaming_system_name",
        "new_streaming_system_type",
        "dc_id",
    ],
)
@click.option(
    "--new-streaming-system-name",
    help="Streaming System Name, if we are creating a new streaming system along the "
    "new cluster connection.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["streaming_system_id"],
)
@click.option(
    "--new-streaming-system-type",
    help="Streaming System Type, if we are creating a new streaming system along the "
    "new cluster connection.",
    required=False,
    type=click.Choice(["confluent-cloud", "msk", "self-hosted"]),
    cls=AdvancedOptions,
    mutually_exclusive_options=["streaming_system_id"],
)
@click.option(
    "--dc-id",
    help="Data Collector UUID. Only specify when there are more than one data collector "
    "in the system, and you are "
    "trying to create a new streaming system for the cluster connection.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["streaming_system_id"],
)
@click.option(
    "--mc-cluster-id",
    help="Existing Streaming Cluster MC UUID. If we are only adding a connection "
    "to a specific cluster, we set this.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=[
        "new_cluster_id",
        "new_cluster_name",
        "new_cluster_type",
    ],
)
@click.option(
    "--new-cluster-id",
    help="Streaming cluster id in your streaming system, required when creating a new cluster",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["mc_cluster_id"],
)
@click.option(
    "--new-cluster-name",
    help="New streaming cluster name at MC side. If not specified, will use "
    "cluster ID as the cluster name when creating a new cluster.",
    required=False,
    cls=AdvancedOptions,
    mutually_exclusive_options=["mc_cluster_id"],
)
def add_streaming_cluster_connection(ctx, **kwargs):
    StreamingOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_streaming_cluster_connection",
    ).onboard_streaming_cluster_connection(**kwargs)


@integrations.command(help="Setup a Pinecone integration.")
@click.pass_obj
@click.option("--environment", help="Pinecone environment (e.g. us-east-1-aws).", required=True)
@click.option("--project-id", help="Pinecone project id.", required=True)
@click.option("--api-key", help="API key for Pinecone project.", required=True)
@click.option(
    "--name",
    help="Friendly name for the integration (defaults to environment:project-id).",
    required=False,
)
@add_common_options(DISAMBIGUATE_DC_OPTIONS)
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def add_pinecone(ctx, **kwargs):
    VectorDbOnboardingService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="integrations add_pinecone",
    ).onboard_pinecone(**kwargs)



import json
import re
import uuid
from dataclasses import dataclass
from datetime import datetime, timedelta, timezone
from typing import (
    Dict,
    List,
    Optional,
)
from urllib.parse import urljoin

import click
import questionary
import requests
from dataclasses_json import (
    DataClassJsonMixin,
    Undefined,
    dataclass_json,
)
from pycarlo.core import (
    Client,
    Mutation,
    Query,
)
from pycarlo.lib.schema import GenericScalar
from sgqlc.types import Variable
from tabulate import tabulate

from montecarlodata import settings
from montecarlodata.agents.fields import (
    AWS,
    AWS_ASSUMABLE_ROLE,
    AZURE_FUNCTION_APP_KEY,
    AZURE_STORAGE_ACCOUNT_KEYS,
    GCP_JSON_SERVICE_ACCOUNT_KEY,
    OAUTH2_CLIENT_CREDENTIALS,
    REMOTE_AGENT,
)
from montecarlodata.common.common import ConditionalDictionary, read_as_json_string
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, manage_errors

DOCKER_TAGS_REQUEST_PAGE_SIZE = 50
DEFAULT_TAGS_PAGE_SIZE = 10
DOCKER_HUB_TAGS_BASE_URL = "https://hub.docker.com/v2/namespaces/"

# Adapted from https://docs.aws.amazon.com/lambda/latest/dg/API_Invoke.html#API_Invoke_RequestSyntax
LAMBDA_ARN_REGEX = re.compile(
    r"^arn:(aws[a-zA-Z-]*)?:lambda:([a-z]{2}(-gov)?-[a-z]+-\d{1}):(\d{12}):function:"
    r"([a-zA-Z0-9-_\.]+)(:(\$LATEST|[a-zA-Z0-9-_]+))?$"
)
LAMBDA_ARN_REGEX_GROUP_REGION = 2
LAMBDA_ARN_REGEX_GROUP_ACCOUNT_ID = 4


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclass()
class DockerTag(DataClassJsonMixin):
    name: str
    last_updated: str


@dataclass_json(undefined=Undefined.EXCLUDE)
@dataclass()
class DockerTagsResult(DataClassJsonMixin):
    results: List[DockerTag]


class AgentService:
    _AGENT_FRIENDLY_HEADERS = [
        "Agent ID",
        "Agent Type / Platform",
        "DC ID",
        "Endpoint",
        "Version",
        "Last updated (UTC)",
        "Active",
    ]

    def __init__(
        self,
        config: Config,
        mc_client: Client,
        command_name: str,
        user_service: Optional[UserService] = None,
    ):
        self._abort_on_error = True
        self._mc_client = mc_client
        self._command_name = command_name
        self._user_service = user_service or UserService(
            config=config,
            command_name=self._command_name,
        )
        self._image_org = config.mcd_agent_image_org
        self._image_repo = config.mcd_agent_image_repo
        self._image_name = f"{config.mcd_agent_image_host}/{self._image_org}/{self._image_repo}"

    @property
    def _ecr_image_name(self) -> str:
        if self._image_name.endswith("/pre-release-agent"):
            return "404798114945.dkr.ecr.*.amazonaws.com/mcd-pre-release-agent"
        return "752656882040.dkr.ecr.*.amazonaws.com/mcd-agent"

    @manage_errors
    def create_agent(self, agent_type, platform, storage, auth_type, endpoint, **kwargs) -> None:
        """
        Register an agent by validating connection and creating an AgentModel in the monolith.
        """

        dry_run = kwargs.get("dry_run", False)
        agent_request = {
            "agent_type": agent_type,
            "endpoint": endpoint,
            "storage_type": storage,
            "platform": platform,
            "auth_type": auth_type,
            "dry_run": dry_run,
        }

        if kwargs.get("dc_id"):
            agent_request["data_collector_id"] = kwargs["dc_id"]

        aws_region: Optional[str] = None
        if platform == AWS and agent_type == REMOTE_AGENT:
            matches = LAMBDA_ARN_REGEX.match(endpoint)
            if matches:
                aws_region = matches.group(LAMBDA_ARN_REGEX_GROUP_REGION)
        if auth_type == GCP_JSON_SERVICE_ACCOUNT_KEY:
            agent_request["credentials"] = read_as_json_string(kwargs["key_file"])
        elif auth_type == AWS_ASSUMABLE_ROLE:
            creds = {"aws_assumable_role": kwargs["assumable_role"]}
            if kwargs["external_id"]:
                creds["external_id"] = kwargs["external_id"]
            if aws_region:
                creds["aws_region"] = aws_region
            agent_request["credentials"] = json.dumps(creds)
        elif auth_type == AZURE_STORAGE_ACCOUNT_KEYS:
            creds = {"azure_connection_string": kwargs["connection_string"]}
            agent_request["credentials"] = json.dumps(creds)
        elif auth_type == AZURE_FUNCTION_APP_KEY:
            creds = {"app_key": kwargs["app_key"]}
            agent_request["credentials"] = json.dumps(creds)
        elif auth_type == OAUTH2_CLIENT_CREDENTIALS:
            creds = {
                "auth_url": kwargs["auth_url"],
                "client_id": kwargs["client_id"],
                "client_secret": kwargs["client_secret"],
            }
            agent_request["credentials"] = json.dumps(creds)

        mutation = Mutation()
        # the trailing call to __fields__ is needed to force selection of all possible fields
        mutation.create_or_update_agent(**agent_request).__fields__()
        result = self._mc_client(mutation).create_or_update_agent

        self._validate_response(result.validation_result)

        if result.agent_id is not None:
            click.echo("Agent successfully registered!\n" f"AgentId: {result.agent_id}")
        elif dry_run:
            if result.validation_result.success:
                click.echo("Dry run completed successfully!")
            else:
                complain_and_abort("Dry run failed.")
        else:
            complain_and_abort("Failed to register agent.")

    @manage_errors
    def delete_agent(self, agent_id) -> None:
        """
        Deregister an Agent (deletes AgentModel from monolith)
        """
        variables = dict(agent_id=agent_id)

        mutation = Mutation()
        mutation.delete_agent(**variables)
        result = self._mc_client(mutation).delete_agent

        if result.success:
            click.echo(f"Agent {agent_id} deregistered.")
        else:
            complain_and_abort("Failed to deregister agent.")

    @manage_errors
    def echo_agents(
        self,
        show_inactive: bool = False,
        headers: str = "firstrow",
        table_format: str = "fancy_grid",
    ):
        """
        Display agents in an easy-to-read table.
        """

        table = [self._AGENT_FRIENDLY_HEADERS]
        for agent in self._user_service.agents:
            is_active = not agent.get("isDeleted")
            if not show_inactive and not is_active:
                continue
            full_type = f"{agent.get('agentType', '')} / {agent.get('platform', '')}"
            last_updated = (
                datetime.fromisoformat(
                    agent.get(
                        "lastUpdatedTime",
                    )  # type: ignore
                ).strftime("%Y-%m-%d %H:%M:%S")
                if agent.get("lastUpdatedTime")
                else "-"
            )

            table += [
                [
                    agent.get("uuid") or "",
                    full_type,
                    agent.get("dc_id") or "",
                    agent.get("endpoint") or "",
                    agent.get("imageVersion") or "-",
                    last_updated,
                    is_active,
                ]
            ]

        # If the account has no agents, add 1 line of empty values so tabulate() creates a pretty
        # empty table
        if len(table) == 1:
            table += ["" for _ in self._AGENT_FRIENDLY_HEADERS]

        click.echo(tabulate(table, headers=headers, tablefmt=table_format, maxcolwidths=100))

    @staticmethod
    def _validate_response(validation_result):
        output = {}

        if validation_result.errors:
            errors = []
            for error in validation_result.errors:
                error_output = dict(
                    message=error.friendly_message or "",
                    cause=error.cause or "",
                    resolution=error.resolution or "",
                )
                if settings.MCD_VERBOSE_ERRORS and error.stack_trace:
                    stack_trace_filename = f"mcd_error_trace_{uuid.uuid4()}.txt"
                    with open(stack_trace_filename, "w") as stack_trace_file:
                        stack_trace_file.write(f"-----------\n{error.stack_trace}\n")
                    error_output["stack_trace"] = f"Stack trace written to {stack_trace_filename}"

                errors.append(error_output)
            output["errors"] = errors
        if validation_result.warnings:
            output["warnings"] = [
                dict(
                    message=warning.friendly_message or "",
                    cause=warning.cause or "",
                    resolution=warning.resolution or "",
                )
                for warning in validation_result.warnings
            ]

        # If there are any errors or warnings returned, display them
        if "errors" in output or "warnings" in output:
            click.echo(json.dumps(output, indent=4))

    @manage_errors
    def check_agent_health(self, agent_id):
        variables = dict(agent_id=agent_id)
        agent = self._user_service.get_agent(agent_id)
        if agent.get("agentType") != "REMOTE_AGENT":  # type: ignore
            storage_query = Query()
            storage_query.test_data_store_reachability(**variables)
            storage_result = self._mc_client(
                storage_query,
                idempotent_request_id=str(uuid.uuid4()),
                timeout_in_seconds=40,  # let Monolith timeout first
            ).test_data_store_reachability

            if not storage_result.success:
                return self._validate_response(storage_result)
            return click.echo("Agent health check succeeded!")
        else:
            variables["validation_name"] = "validate_agent_reachability"
            agent_reachability_query = Query()
            agent_reachability_query.test_agent_reachability(**variables)
            agent_reachability_result = self._mc_client(
                agent_reachability_query,
                idempotent_request_id=str(uuid.uuid4()),
                timeout_in_seconds=40,  # let Monolith timeout first
            ).test_agent_reachability

            if not agent_reachability_result.success:
                return self._validate_response(agent_reachability_result)

            variables["validation_name"] = "validate_storage_access"
            agent_storage_query = Query()
            agent_storage_query.test_agent_reachability(**variables)
            agent_storage_result = self._mc_client(
                agent_storage_query,
                idempotent_request_id=str(uuid.uuid4()),
                timeout_in_seconds=40,  # let Monolith timeout first
            ).test_agent_reachability

            if not agent_storage_result.success:
                return self._validate_response(agent_storage_result)

            if agent_reachability_result.additional_data.returned_data:
                click.echo(
                    json.dumps(
                        agent_reachability_result.additional_data.returned_data,
                        indent=4,
                    )
                )
            return click.echo("Agent health check succeeded!")

    @manage_errors
    def upgrade_agent(self, parameters: Optional[Dict] = None, **kwargs):
        image = self._choose_image(**kwargs)
        click.echo(f"Upgrading agent with image '{image}'")
        variables = {
            "agent_id": kwargs["agent_id"],
            "image": image,
            "parameters": Variable("parameters"),
        }

        mutation = Mutation(parameters=GenericScalar)
        mutation.upgrade_agent(**variables)
        result = self._mc_client(
            mutation,
            variables={
                "parameters": parameters,
            },
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
        ).upgrade_agent

        click.echo("Upgrade succeeded!")
        if result.upgrade_result:
            click.echo(json.dumps(result.upgrade_result, indent=4))

    @manage_errors
    def echo_operation_logs(self, **kwargs):
        parameters = ConditionalDictionary(lambda x: x is not None)
        parameters.update(kwargs)  # don't send null values
        logs_query = Query()
        logs_query.get_agent_operation_logs(**parameters)
        result = self._mc_client(
            logs_query,
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
        )
        logs_results = result.get_agent_operation_logs

        events = [
            {
                "timestamp": event.timestamp,
                "commands": event.payload.get("commands") if event.payload else {},  # type: ignore
                "operation_name": (event.payload.get("operation_name") if event.payload else None),  # type: ignore
                "trace_id": event.payload.get("trace_id") if event.payload else None,  # type: ignore
            }
            for event in logs_results
        ]
        click.echo(json.dumps(events, indent=4))

    @manage_errors
    def echo_aws_upgrade_logs(self, start_time: Optional[str], **kwargs):
        parameters = ConditionalDictionary(lambda x: x is not None)
        parameters["start_time"] = (
            start_time
            or (datetime.now() - timedelta(hours=12)).astimezone(timezone.utc).isoformat()
        )
        parameters.update(kwargs)  # don't send null values
        query = Query()
        query.get_aws_agent_upgrade_logs(**parameters)
        result = self._mc_client(
            query,
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
        )
        results = result.get_aws_agent_upgrade_logs

        events = [
            {
                "timestamp": event.timestamp,
                "logical_resource_id": event.logical_resource_id,
                "resource_status": event.resource_status,
                "resource_status_reason": event.resource_status_reason,
                "resource_type": event.resource_type,
            }
            for event in results
        ]
        click.echo(json.dumps(events, indent=4))

    @manage_errors
    def echo_aws_template(self, **kwargs):
        infra_details = self._get_aws_infra_details(**kwargs)
        click.echo(infra_details.get("template"))

    @manage_errors
    def echo_aws_template_parameters(self, **kwargs):
        infra_details = self._get_aws_infra_details(**kwargs)
        parameters = infra_details.get("parameters")
        click.echo(json.dumps(parameters, indent=4))

    def _get_aws_infra_details(self, **kwargs) -> Dict:
        parameters = ConditionalDictionary(lambda x: x is not None)
        parameters.update(kwargs)  # don't send null values
        query = Query()
        query.get_aws_agent_infra_details(**parameters)
        result = self._mc_client(
            query,
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
        )
        result = result.get_aws_agent_infra_details
        return {
            "template": result.template,
            "parameters": result.parameters,
        }

    def _choose_image(self, agent_id: str, image_tag: Optional[str] = None) -> Optional[str]:
        platform = self._user_service.get_agent(agent_id).get("platform")  # type: ignore

        if not image_tag or not self._validate_tag(image_tag):
            versions = self._get_recent_versions(platform)  # type: ignore
            image_tag = (
                versions[0]
                if not image_tag
                else questionary.select("Please choose a valid image tag", choices=versions).ask()
            )

        if not image_tag:
            # user canceled prompt without choosing a response
            raise click.Abort()

        if platform == "AWS":
            if image_tag.endswith("-lambda"):
                image_tag = image_tag[: -len("-lambda")]
            return f"{self._ecr_image_name}:{image_tag}"
        return f"{self._image_name}:{image_tag}"

    def _validate_tag(self, image_tag: str) -> bool:
        image_path = f"{self._image_org}/repositories/{self._image_repo}"
        url: str = urljoin(DOCKER_HUB_TAGS_BASE_URL, f"{image_path}/tags/{image_tag}")
        response = requests.head(url=url)
        return response.status_code == 200

    def _get_recent_versions(self, platform: str) -> List[str]:
        image_path = f"{self._image_org}/repositories/{self._image_repo}"
        url = urljoin(
            DOCKER_HUB_TAGS_BASE_URL,
            f"{image_path}/tags?page_size={DOCKER_TAGS_REQUEST_PAGE_SIZE}&page=1",
        )
        response = requests.get(url=url)
        response.raise_for_status()
        payload: DockerTagsResult = DockerTagsResult.from_json(response.text)
        image_variant = (
            "cloudrun"
            if platform == "GCP"
            else ("azure" if platform == "AZURE" else "lambda" if platform == "AWS" else "generic")
        )
        click.echo(f"Defaulting to {image_variant} image variants because platform is {platform}")
        payload.results = list(
            filter(
                lambda t: image_variant in t.name and "latest" not in t.name,
                payload.results,
            )
        )[:DEFAULT_TAGS_PAGE_SIZE]
        payload.results.sort(key=lambda t: t.last_updated, reverse=True)
        return [tag.name for tag in payload.results]



# Queries related to managing the collector

GENERATE_COLLECTOR_TEMPLATE = """
mutation generateCollectorTemplate($region:String, $dcId:UUID, $updateInfra:Boolean) {
  generateCollectorTemplate(region:$region, dcId:$dcId, updateInfra: $updateInfra) {
    dc {
      uuid
      templateLaunchUrl
      stackArn
      customerAwsAccountId
      active
      apiGatewayId
      templateVariant
    }
  }
}
"""

ADD_COLLECTOR_RECORD = """
mutation createCollectorRecord {
  createCollectorRecord {
    dc {
      uuid
    }
  }
}
"""

TEST_TELNET_CONNECTION = """
query testTelnetConnection($host:String, $port:Int, $timeout:Int, $dcId:UUID) {
  testTelnetConnection(host:$host, port:$port, timeout:$timeout, dcId:$dcId) {
    success
    validations {
      type
      message
    }
  }
}
"""

TEST_TCP_OPEN_CONNECTION = """
query testTcpOpenConnection($host:String, $port:Int, $timeout:Int, $dcId:UUID) {
  testTcpOpenConnection(host:$host, port:$port, timeout:$timeout, dcId:$dcId) {
    success
    validations {
      type
      message
    }
  }
}
"""



import base64
import ipaddress
import json
import pkgutil
import secrets
from collections.abc import Mapping, MutableMapping
from functools import wraps
from typing import Any, Callable, Dict, Iterator, List, Optional

from box import Box
from jinja2 import Template
from pycarlo.core import Client, Session

from montecarlodata.config import Config


class ConditionalDictionary(MutableMapping):
    """
    A dictionary where you can specify a condition
    when initially assigning a value (e.g. no null values)
    """

    def __init__(self, condition: Callable):
        self.condition = condition
        self.mapping = {}

    def __setitem__(self, key, value) -> None:
        if key in self.mapping or self.condition(value):
            self.mapping[key] = value

    def __delitem__(self, key) -> None:
        del self.mapping[key]

    def __getitem__(self, key):
        return self.mapping[key]

    def __len__(self) -> int:
        return len(self.mapping)

    def __iter__(self) -> Iterator:
        return iter(self.mapping)


def normalize_gql(field: str) -> Optional[str]:
    if field:
        return field.replace("_", "-").lower()


def read_as_base64(path: str) -> bytes:
    with open(path, "rb") as fp:
        return base64.b64encode(fp.read())


def read_as_json(path: str) -> Dict:
    with open(path) as file:
        return json.load(file)


def read_as_json_string(path: str) -> str:
    """ "Read and validate JSON file"""
    return json.dumps(read_as_json(path))


def struct_match(s1: Dict, s2: Dict) -> bool:
    return json.dumps(s1, sort_keys=True) == json.dumps(s2, sort_keys=True)


def boxify(
    use_snakes: bool = False,
    default_box_attr: Optional[Any] = object(),
    default_box: bool = False,
):
    """
    Convenience decorator to convert a dict into Box for ease of use.

    Set `use_snakes` to convert camelCase to snake_case. Use `default_box_attr`
    to set a default value.
    """

    def _boxify(func):
        @wraps(func)
        def _impl(self, *args, **kwargs):
            dict_ = func(self, *args, **kwargs)
            if dict_ and isinstance(dict_, Mapping):
                return Box(
                    dict_,
                    camel_killer_box=use_snakes,
                    default_box_attr=default_box_attr,
                    default_box=default_box,
                )
            return dict_

        return _impl

    return _boxify


def chunks(lst: List, n: int):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def is_overlap(r1: str, r2: str) -> bool:
    """Check if two CIDR ranges overlap"""
    net1 = ipaddress.ip_network(r1)
    net2 = ipaddress.ip_network(r2)
    return net1.overlaps(net2) or net2.overlaps(net1)


def create_mc_client(ctx: Dict) -> Client:
    config: Config = ctx["config"]
    mc_client = Client(
        session=Session(
            endpoint=config.mcd_api_endpoint,
            mcd_id=config.mcd_id,
            mcd_token=config.mcd_token,
        )
    )
    return mc_client


def read_files(files: List) -> Iterator[str]:
    """Read a list of files"""
    for fp in files:
        yield fp.read()


def render_dumped_json(path: str, **kwargs) -> str:
    """Render and dump as formatted JSON"""
    return json.dumps(json.loads(render(path, **kwargs)), indent=4, sort_keys=True)


def render(path: str, **kwargs) -> str:
    """Load file from path and inject kwargs."""
    template = Template(pkgutil.get_data(__name__, path).decode("utf-8"))  # type: ignore
    return template.render(**kwargs)


def generate_token(length: Optional[int] = 16) -> str:
    """Generate a random url safe token"""
    return secrets.token_urlsafe(length)



import click

from montecarlodata.common import create_mc_client
from montecarlodata.management.service import ManagementService
from montecarlodata.tools import AdvancedOptions, convert_empty_str_callback


@click.group(help="Manage account settings.")
def management():
    """
    Group for any management related subcommands
    """
    pass


@management.command(help="Get PII filtering preferences.")
@click.pass_obj
def get_pii_preferences(ctx):
    ManagementService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="management get_pii_preferences",
    ).get_pii_preferences()


@management.command(help="Configure PII filtering preferences for the account.")
@click.option(
    "--enable/--disable",
    "enabled",
    required=False,
    type=click.BOOL,
    default=None,
    help="Whether PII filtering should be active for the account.",
)
@click.option(
    "--fail-mode",
    required=False,
    type=click.Choice(["CLOSE", "OPEN"], case_sensitive=False),
    help="Whether PII filter failures will allow (OPEN) or prevent (CLOSE) data "
    "flow for this account.",
)
@click.pass_obj
def configure_pii_filtering(ctx, **kwargs):
    ManagementService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="management configure_pii_filtering",
    ).set_pii_filtering(**kwargs)


@management.command(help="List entities blocked from collection on this account.")
@click.option(
    "--resource-name",
    required=False,
    help="Name of a specific resource to filter by. Shows all resources by default.",
)
@click.pass_obj
def get_collection_block_list(ctx, **kwargs):
    ManagementService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="management get_collection_block_list",
    ).get_collection_block_list(**kwargs)


@management.command(help="Update entities for which collection is blocked on this account.")
@click.option(
    "--add/--remove",
    "adding",
    required=True,
    type=click.BOOL,
    default=None,
    help="Whether the entities being specified should be added or removed from the block list.",
)
@click.option(
    "--resource-name",
    "resource_name",
    help="Name of a specific resource to apply collection block to. "
    "Only warehouse names are supported for now.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["filename"],
    required_with_options=["project"],
    at_least_one_set=["resource_name", "filename"],
)
@click.option(
    "--project",
    "project",
    help="Top-level object hierarchy e.g. database, catalog, etc.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["filename"],
    required_with_options=["resource_name"],
)
@click.option(
    "--dataset",
    "dataset",
    default=None,
    callback=convert_empty_str_callback,
    required=False,
    help="Intermediate object hierarchy e.g. schema, database, etc.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["filename"],
    required_with_options=["resource_name", "project"],
)
@click.option(
    "--collection-block-list-filename",
    "filename",
    help="Filename that contains collection block definitions. "
    "This file is expected to be in a CSV format with the headers resource_name, project, "
    "and dataset.",
    cls=AdvancedOptions,
    mutually_exclusive_options=["resource_name", "project", "dataset"],
)
@click.pass_obj
def update_collection_block_list(ctx, **kwargs):
    ManagementService(
        config=ctx["config"],
        mc_client=create_mc_client(ctx),
        command_name="management update_collection_block_list",
    ).update_collection_block_list(**kwargs)



from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.glue_athena import (
    GlueAthenaOnboardingService,
)
from montecarlodata.queries.onboarding import (
    TEST_ATHENA_CRED_MUTATION,
    TEST_GLUE_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class GlueAthenaOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = GlueAthenaOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(GlueAthenaOnboardingService, "onboard")
    def test_glue_flow(self, onboard_mock):
        self._service.onboard_glue(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_GLUE_CRED_MUTATION,
            validation_response="testGlueCredentials",
            connection_type="glue",
            **_SAMPLE_BASE_OPTIONS,
        )

    @patch.object(GlueAthenaOnboardingService, "onboard")
    def test_athena_flow(self, onboard_mock):
        self._service.onboard_athena(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_ATHENA_CRED_MUTATION,
            validation_response="testAthenaCredentials",
            connection_type="athena",
            **_SAMPLE_BASE_OPTIONS,
        )



import copy
import json
import re
import uuid
from concurrent.futures import ThreadPoolExecutor, as_completed
from typing import Any, Dict, List, Optional, Tuple, Union

import click
from box import Box
from pycarlo.common.errors import GqlError
from pycarlo.core import Client, Query
from pycarlo.lib.schema import TestCredentialsV2Response

from montecarlodata.common.echo_utils import (
    echo_error_message,
    echo_success_message,
    echo_warning_message,
    styled_error_icon,
    styled_success_icon,
    styled_warning_icon,
)
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, prompt_connection
from montecarlodata.integrations.onboarding.fields import (
    AZURE_DATA_FACTORY_CONNECTION_TYPE,
    AZURE_DEDICATED_SQL_POOL_TYPE,
    AZURE_SQL_DATABASE_TYPE,
    CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE,
    CONFLUENT_KAFKA_CONNECTION_TYPE,
    DREMIO_DATABASE_TYPE,
    EXPECTED_TEST_AZURE_DATA_FACTORY_RESPONSE_FIELD,
    EXPECTED_TEST_CONFLUENT_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_CONFLUENT_KAFKA_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_INFORMATICA_CREDENTIALS_RESPONSE_FIELD,
    EXPECTED_TEST_MSK_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_MSK_KAFKA_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_PINECONE_CREDENTIALS_RESPONSE_FIELD,
    EXPECTED_TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_SELF_HOSTED_KAFKA_CRED_RESPONSE_FIELD,
    EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
    INFORMATICA_CONNECTION_TYPE,
    MARIADB_DB_TYPE,
    MOTHERDUCK_DATABASE_TYPE,
    MSK_KAFKA_CONNECT_CONNECTION_TYPE,
    MSK_KAFKA_CONNECTION_TYPE,
    MYSQL_DB_TYPE,
    ORACLE_DB_TYPE,
    PINECONE_CONNECTION_TYPE,
    POSTGRES_DB_TYPE,
    SAP_HANA_DATABASE_TYPE,
    SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE,
    SELF_HOSTED_KAFKA_CONNECTION_TYPE,
    SQL_SERVER_DB_TYPE,
    TERADATA_DB_TYPE,
    TRANSACTIONAL_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    TEST_AZURE_DATA_FACTORY_CRED_MUTATION,
    TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
    TEST_CONFLUENT_KAFKA_CRED_MUTATION,
    TEST_INFORMATICA_CRED_MUTATION,
    TEST_MSK_KAFKA_CONNECT_CRED_MUTATION,
    TEST_MSK_KAFKA_CRED_MUTATION,
    TEST_PINECONE_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
    TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
    TEST_TRANSACTIONAL_DB_CRED_MUTATION,
)
from montecarlodata.utils import GqlWrapper


class CollectorValidationService:
    _SAVE_CREDENTIALS = "save_credentials"

    _CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CRED_MUTATION,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: TEST_CONFLUENT_KAFKA_CONNECT_CRED_MUTATION,
        MSK_KAFKA_CONNECTION_TYPE: TEST_MSK_KAFKA_CRED_MUTATION,
        MSK_KAFKA_CONNECT_CONNECTION_TYPE: TEST_MSK_KAFKA_CONNECT_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CRED_MUTATION,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_MUTATION,
        PINECONE_CONNECTION_TYPE: TEST_PINECONE_CRED_MUTATION,
        TRANSACTIONAL_CONNECTION_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        TERADATA_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        POSTGRES_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        SQL_SERVER_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        MYSQL_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        ORACLE_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        MARIADB_DB_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        AZURE_DEDICATED_SQL_POOL_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        AZURE_SQL_DATABASE_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        SAP_HANA_DATABASE_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        MOTHERDUCK_DATABASE_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        DREMIO_DATABASE_TYPE: TEST_TRANSACTIONAL_DB_CRED_MUTATION,
        INFORMATICA_CONNECTION_TYPE: TEST_INFORMATICA_CRED_MUTATION,
        AZURE_DATA_FACTORY_CONNECTION_TYPE: TEST_AZURE_DATA_FACTORY_CRED_MUTATION,
    }

    _CONNECTION_TYPES_TO_OPERATION_TYPE = {
        CONFLUENT_KAFKA_CONNECTION_TYPE: EXPECTED_TEST_CONFLUENT_KAFKA_CRED_RESPONSE_FIELD,
        CONFLUENT_KAFKA_CONNECT_CONNECTION_TYPE: EXPECTED_TEST_CONFLUENT_KAFKA_CONNECT_CRED_RESPONSE_FIELD,  # noqa
        MSK_KAFKA_CONNECTION_TYPE: EXPECTED_TEST_MSK_KAFKA_CRED_RESPONSE_FIELD,
        MSK_KAFKA_CONNECT_CONNECTION_TYPE: EXPECTED_TEST_MSK_KAFKA_CONNECT_CRED_RESPONSE_FIELD,
        SELF_HOSTED_KAFKA_CONNECTION_TYPE: EXPECTED_TEST_SELF_HOSTED_KAFKA_CRED_RESPONSE_FIELD,
        SELF_HOSTED_KAFKA_CONNECT_CONNECTION_TYPE: EXPECTED_TEST_SELF_HOSTED_KAFKA_CONNECT_CRED_RESPONSE_FIELD,  # noqa
        PINECONE_CONNECTION_TYPE: EXPECTED_TEST_PINECONE_CREDENTIALS_RESPONSE_FIELD,
        TRANSACTIONAL_CONNECTION_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        TERADATA_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        POSTGRES_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        SQL_SERVER_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        MYSQL_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        ORACLE_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        MARIADB_DB_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        AZURE_DEDICATED_SQL_POOL_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        AZURE_SQL_DATABASE_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        SAP_HANA_DATABASE_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        MOTHERDUCK_DATABASE_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        DREMIO_DATABASE_TYPE: EXPECTED_TEST_TRANSACTIONAL_DB_CREDENTIALS_RESPONSE_FIELD,
        INFORMATICA_CONNECTION_TYPE: EXPECTED_TEST_INFORMATICA_CREDENTIALS_RESPONSE_FIELD,
        AZURE_DATA_FACTORY_CONNECTION_TYPE: EXPECTED_TEST_AZURE_DATA_FACTORY_RESPONSE_FIELD,
    }

    def __init__(
        self,
        config: Config,
        mc_client: Client,
        command_name: str,
        request_wrapper: Optional[GqlWrapper] = None,
        user_service: Optional[UserService] = None,
    ):
        self._mc_client = mc_client
        self._command_name = command_name
        self._request_wrapper = request_wrapper or GqlWrapper(
            config,
            command_name=self._command_name,
        )
        self._user_service = user_service or UserService(
            request_wrapper=self._request_wrapper,
            config=config,
            command_name=self._command_name,
        )

    def run_validations(
        self,
        dc_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        only_periodic: bool = False,
    ) -> int:
        """
        Runs all validators for all integrations in the data collector.

        :param dc_id: The optional UUID of the data collector, used to disambiguate.
        :param agent_id: The optional UUID of the agent, used to disambiguate.
        :param only_periodic: Whether only periodic validations must be run or not.
        """
        if agent_id:
            agent = self._user_service.get_agent(agent_id)
            dc_id = agent["dc_id"]  # type: ignore
        dc = self._user_service.get_collector(dc_id)

        # Filter integration list to just integrations for this dc
        all_integrations = [
            integration
            for integration in (
                (self._user_service.warehouses or [])
                + (self._user_service.bi_containers or [])
                + (self._user_service.etl_containers or [])
            )
            if integration.get("dataCollector", {}).get("uuid") == dc.uuid
        ]

        total_failures = 0

        executor = ThreadPoolExecutor(max_workers=4)
        # Start the load operations and mark each future with its URL
        future_to_integration = {
            executor.submit(
                self._run_validations_for_integration, dc, integration, only_periodic
            ): integration
            for integration in all_integrations
        }
        for future in as_completed(future_to_integration):
            integration = future_to_integration[future]
            try:
                integration_failures, integration_messages = future.result()
                total_failures += integration_failures
                for message in integration_messages:
                    click.echo(message)
            except Exception as exc:
                print("%r generated an exception: %s" % (integration, exc))

        # Run storage access validation. If DC has an agent registered, will validate that the
        # agent's storage location is accessible.
        # If DC has no agent registered, will validate that default storage location is accessible.
        click.echo("")
        total_failures += self._run_storage_access_validation(dc_id=dc.uuid)

        click.echo("")
        if total_failures == 0:
            click.echo("All validations passed")
        else:
            click.echo(f"{total_failures} validations failed")
        return total_failures

    def _run_validations_for_integration(
        self, dc: Box, integration: Dict, only_periodic: bool
    ) -> Tuple[int, List[str]]:
        integration_failures = 0
        click.echo(f"Running validations for {integration['name']}")
        messages = []
        connections: List[Dict[str, Any]] = integration.get("connections", [])
        for connection in connections:
            if "type" in connection and "uuid" in connection:
                (
                    connection_failures,
                    connection_messages,
                ) = self._run_connection_validators(
                    dc_uuid=dc.uuid,
                    name=integration.get("name", ""),
                    connection_type=connection["type"].lower(),
                    connection_uuid=connection["uuid"],
                    only_periodic=only_periodic,
                )
                integration_failures += connection_failures
                messages.append("")
                messages.extend(connection_messages)

        return integration_failures, messages

    def _run_connection_validators(
        self,
        dc_uuid: uuid.UUID,
        name: str,
        connection_type: str,
        connection_uuid: uuid.UUID,
        only_periodic: bool,
    ) -> Tuple[int, List[str]]:
        messages = []
        messages.append(f"Validating {connection_type} connection: {name} - {connection_uuid}")
        validations = self._get_supported_validations(
            dc_uuid=dc_uuid,
            connection_type=connection_type,
            only_periodic=only_periodic,
        )
        failures = 0
        for validation in validations:
            ok = False
            warning = False
            try:
                result = self._run_single_validation(
                    connection_uuid=connection_uuid,
                    validation_name=validation,
                )

                assert result is not None
                ok, warning = self._process_validation_result(result)
            except Exception as e:
                echo_error_message(f"\tValidation {validation} failed ({e}).")

            result_icon, failures = self._get_icon(ok, warning, failures)
            messages.append(f"\t{validation}: {result_icon}")

        return failures, messages

    def _get_supported_validations(
        self,
        dc_uuid: Union[uuid.UUID, str],
        connection_type: str,
        only_periodic: bool,
    ) -> List:
        query = Query()
        query.get_supported_validations_v2(dc_id=dc_uuid, connection_type=connection_type)

        try:
            result = self._mc_client(
                query=query,
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "collector_validation",
                    "x-mcd-telemetry-command": self._command_name,
                },
            ).get_supported_validations_v2

            return [
                validation.name
                for validation in result.supported_validations
                if not only_periodic or validation.periodic_validation
            ]
        except GqlError as e:
            complain_and_abort(f"Unable to get list of supported validators ({e}).")
            return []

    def _run_single_validation(
        self,
        connection_uuid: uuid.UUID,
        validation_name: str,
    ) -> Optional[TestCredentialsV2Response]:
        query = Query()
        query.test_existing_connection_v2(
            connection_id=connection_uuid, validation_name=validation_name
        )

        return self._mc_client(
            query=query,
            idempotent_request_id=str(uuid.uuid4()),
            timeout_in_seconds=40,  # let Monolith timeout first
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "collector_validation",
                "x-mcd-telemetry-command": self._command_name,
            },
        ).test_existing_connection_v2  # type: ignore

    def _run_storage_access_validation(
        self,
        dc_id: uuid.UUID,
    ) -> int:
        query = Query()
        query.test_storage_access(dc_id=dc_id)

        click.echo("Validating storage access:")

        failures = 0
        ok = False
        warning = False
        try:
            result = self._mc_client(
                query=query,
                idempotent_request_id=str(uuid.uuid4()),
                timeout_in_seconds=40,  # let Monolith timeout first
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "collector_validation",
                    "x-mcd-telemetry-command": self._command_name,
                },
            ).test_storage_access

            ok, warning = self._process_validation_result(result)  # type: ignore
        except Exception as e:
            echo_error_message(f"\tValidation validate_storage_access failed ({e}).")

        result_icon, failures = self._get_icon(ok, warning, failures)
        click.echo(f"\tvalidate_storage_access: {result_icon}")

        return failures

    @staticmethod
    def _process_validation_result(result: TestCredentialsV2Response) -> Tuple[bool, bool]:
        ok = False
        warning = False

        # if there are only warnings we log them but consider it a successful validation
        ok = result.success or not bool(result.errors)
        if result.warnings:
            for warning in result.warnings:  # type: ignore
                echo_warning_message(f"\t{warning.cause}")
        if result.errors:
            for error in result.errors:  # type: ignore
                echo_error_message(f"\t{error.cause}")
        elif not result.success:
            warning = True

        return bool(ok), warning

    @staticmethod
    def _get_icon(ok: bool, warning: bool, failures: int) -> Tuple[str, bool]:
        if ok:
            result_icon = styled_warning_icon() if warning else styled_success_icon()
        else:
            result_icon = styled_error_icon()
            failures += 1
        return result_icon, bool(failures)

    def test_new_credentials(
        self,
        connection_type: str,
        dc_id: Optional[uuid.UUID] = None,
        generate_key: bool = True,
        skip_validation: bool = False,
        validate_only: bool = False,
        **kwargs: Any,
    ) -> Optional[str]:
        if not dc_id:
            dc_id = self._user_service.get_collector().uuid
        assert dc_id is not None

        kwargs["dc_id"] = str(dc_id)

        if not skip_validation:
            supported_validation_tests = self._get_supported_validations(
                dc_uuid=str(dc_id),
                connection_type=connection_type,
                only_periodic=False,
            )
            for validation_name in supported_validation_tests:
                variables = copy.deepcopy(kwargs)
                variables["validation_name"] = validation_name

                operation_name = self._CONNECTION_TYPES_TO_OPERATION_TYPE[connection_type]
                operation_name_snake = self._camel_to_snake(operation_name)
                validation_result = (
                    self._mc_client(
                        query=self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING[connection_type],
                        operation_name=operation_name,
                        variables=GqlWrapper.convert_snakes_to_camels(variables),
                        idempotent_request_id=str(uuid.uuid4()),
                        timeout_in_seconds=40,  # let Monolith timeout first
                        additional_headers={
                            "x-mcd-telemetry-reason": "cli",
                            "x-mcd-telemetry-service": "collector_validation",
                            "x-mcd-telemetry-command": self._command_name,
                        },
                    )
                    .__getattr__(operation_name_snake)  # type: ignore
                    .validation_result
                )
                ok, warning = self._process_validation_result(
                    result=validation_result,  # type: ignore
                )

                result_icon, _ = self._get_icon(ok, warning, 0)
                click.echo(f"\t{validation_name}: {result_icon}")

                if not ok:
                    # validation failed
                    raise click.Abort()

        if not validate_only:
            prompt_connection(
                message="Validations passed! Would you like to continue?",
                skip_prompt=not generate_key,
            )
            variables = copy.deepcopy(kwargs)
            variables["validation_name"] = self._SAVE_CREDENTIALS
            new_key_response = self._request_wrapper.make_request_v2(
                query=self._CONNECTION_TYPES_TO_CREDS_MUTATIONS_MAPPING[connection_type],
                operation=self._CONNECTION_TYPES_TO_OPERATION_TYPE[connection_type],
                service="collector_validation",
                variables=variables,
            )

            validation_result = new_key_response.data.validationResult  # type: ignore
            if not validation_result.success:
                click.echo("creating credential key failed.")
                click.echo(json.dumps(validation_result))
                click.Abort()

            return new_key_response.data.key  # type: ignore
        else:
            echo_success_message(message="Validations passed!")

    @staticmethod
    def _camel_to_snake(text: str) -> str:
        return re.sub("([A-Z]+)", r"_\1", text).lower()



from dataclasses import dataclass
from typing import Dict, List, Optional, Union
from uuid import UUID

from box import Box

from montecarlodata.common.common import boxify
from montecarlodata.config import Config
from montecarlodata.errors import (
    AMBIGUOUS_AGENT_OR_COLLECTOR_MESSAGE,
    complain_and_abort,
)
from montecarlodata.queries.user import GET_USER_QUERY
from montecarlodata.utils import GqlWrapper


@dataclass
class User:
    first_name: Optional[str] = None
    last_name: Optional[str] = None


class UserService:
    def __init__(
        self,
        config: Config,
        command_name: str,
        request_wrapper: Optional[GqlWrapper] = None,
    ):
        self._config = config
        self._command_name = command_name
        self._request_wrapper = request_wrapper or GqlWrapper(config, command_name=command_name)
        self._user = self._request_wrapper.make_request(
            query=GET_USER_QUERY,
            service="user_service",
        )

    @property
    def user(self) -> User:
        """
        Returns basic user properties
        """
        return User(
            first_name=self._user["getUser"].get("firstName"),  # type: ignore
            last_name=self._user["getUser"].get("lastName"),  # type: ignore
        )

    @property
    def account(self) -> Dict:
        """
        Get account details
        """
        return self._user["getUser"]["account"]  # type: ignore

    @property
    def collectors(self) -> Dict:
        """
        Get collectors in the account
        """
        return self.account.get("dataCollectors", [{}])

    @property
    def agents(self) -> List[Dict]:
        """
        Get agents in the account
        """
        agent_list = []
        for dc in self.collectors:
            for agent in dc.get("agents"):
                agent["dc_id"] = dc.get("uuid", "")
                agent_list.append(agent)
        return agent_list

    @property
    def active_collector(self) -> Dict:
        """
        Get active collector from collectors.

        Errors out on accounts with > 1 collector. Legacy. Do not use. See `get_collector`
        """
        return self.collectors[self._get_active_collector()]

    @property
    def resource_identifiers(self) -> Dict:
        """
        Returns a commutative mapping of resource uuid and name.
        e.g. user_service.resource_identifiers().get("resource_name") == "resource_uuid"
         and user_service.resource_identifiers().get("resource_uuid") == "resource_name"

        Currently only supports warehouse resources.
        """
        resource_identifiers = {}
        for resource in self.warehouses:
            resource_identifiers[resource["uuid"]] = resource["name"]
            resource_identifiers[resource["name"]] = resource["uuid"]
        return resource_identifiers

    @property
    def warehouses(self) -> List[Dict]:
        """
        Get warehouses attached to the account
        """
        return self.account.get("warehouses")  # type: ignore

    @property
    def bi_containers(self) -> List[Dict]:
        """
        Get bi attached to the account
        """
        return self.account.get("bi")  # type: ignore

    @property
    def etl_containers(self) -> List[Dict]:
        """
        Get etl attached to the account
        """
        return self.account.get("etlContainers")  # type: ignore

    @property
    def tableau_accounts(self) -> Dict:
        """
        Get tableau connections attached to the account
        """
        return self.account.get("tableauAccounts")  # type: ignore

    @property
    def active_collection_regions(self) -> List[str]:
        """
        Get a list of active collection regions
        """
        return self.account.get("activeCollectionRegions")  # type: ignore

    @boxify()
    def get_collector(self, dc_id: Optional[str] = None, agent_id: Optional[str] = None) -> Box:
        """
        Get a specific collector

        This is only necessary for client only ops on a specific collector.
        APIs handle this disambiguation already.
        """
        num_of_collectors = len(self.collectors)
        if num_of_collectors == 0:
            complain_and_abort("No collector found.")

        if agent_id:
            agent = self.get_agent(agent_id)
            dc_id = agent["dc_id"]  # type: ignore

        if dc_id:
            for collector in self.collectors:
                if dc_id == collector["uuid"]:
                    return collector
            complain_and_abort(f"Collector with ID '{dc_id}' not found.")
        if num_of_collectors > 1:
            complain_and_abort(AMBIGUOUS_AGENT_OR_COLLECTOR_MESSAGE)
        return self.collectors[0]

    def get_agent(self, agent_id: str) -> Optional[Dict]:
        if len(self.agents) == 0:
            complain_and_abort("No agents found.")

        for agent in self.agents:
            if agent.get("isDeleted") != "true" and agent_id == agent["uuid"]:
                return agent
        complain_and_abort(f"Agent with ID '{agent_id}' not found.")

    def get_collector_agent(self, dc_id: Optional[str] = None) -> Optional[Box]:
        dc = self.get_collector(dc_id)
        for agent in dc.agents:
            if not agent.isDeleted:
                return agent

    def _get_active_collector(self) -> Optional[int]:
        """
        Get active collector - currently only one active collector per account is supported.
        Abort if None are found
        """
        if len(self.collectors) > 1:
            complain_and_abort("This option is only supported in accounts with one collector.")

        for idx, collector in enumerate(self.collectors):
            if collector.get("active"):
                return idx
        complain_and_abort("No active collector found")

    def get_warehouse_for_connection(self, connection_id: Union[str, UUID]) -> Optional[Box]:
        """
        Get warehouse for the given connection id
        """
        for warehouse in self.warehouses:
            for connection in warehouse.get("connections", []):
                if str(connection_id) == connection.get("uuid"):
                    return Box(warehouse)

    def get_warehouses_with_connection_type(
        self, connection_type: str, warehouse_name: Optional[str] = None
    ) -> List[Box]:
        results = []
        for warehouse in self.warehouses:
            if warehouse_name is None or warehouse_name == warehouse.get("name"):
                for connection in warehouse.get("connections", []):
                    if connection_type == connection.get("type"):
                        results.append(Box(warehouse))
        return results
